@@ -1,7 +1,7 @@
 /*
  * ica_main.cpp - main-file for ICA (iTALC Client Application)
  *
- * Copyright (c) 2006-2007 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ * Copyright (c) 2006-2010 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
  *  
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -28,17 +28,16 @@
 #endif
 
 #include <QtCore/QLocale>
-#include <QtCore/QTranslator>
 #include <QtGui/QApplication>
 #include <QtNetwork/QHostInfo>
 
 
 #include "ica_main.h"
 #include "system_service.h"
-#include "remote_control_widget.h"
 #include "isd_server.h"
 #include "ivs.h"
-#include "local_system.h"
+#include "local_system_ica.h"
+#include "lock_widget.h"
 #include "debug.h"
 #include "system_key_trapper.h"
 #include "dsa_key.h"
@@ -102,7 +101,9 @@ int ICAMain( int argc, char * * argv )
 	{
 		if( QString( argv[i] ) == "-rx11vs" ||
 			QString( argv[i] ) == "-createkeypair" ||
-			QString( argv[i] ) == "-h" )
+			QString( argv[i] ) == "-h" ||
+			QString( argv[i] ) == "--version" ||
+			QString( argv[i] ) == "-v" )
 		{
 			core_app = TRUE;
 		}
@@ -111,12 +112,7 @@ int ICAMain( int argc, char * * argv )
 	if( !core_app )
 	{
 		systemService s( "icas", SERVICE_ARG, __app_name,
-#ifndef NO_LUPUS_INTEGRATION
-				"lupus",
-#else
-				"",
-#endif
-						serviceMain, argc, argv );
+					"", serviceMain, argc, argv );
 		if( s.evalArgs( argc, argv ) || argc == 0 )
 		{
 			return( 0 );
@@ -133,6 +129,11 @@ int ICAMain( int argc, char * * argv )
 	{
 		QApplication * a = new QApplication( argc, argv );
 		a->setQuitOnLastWindowClosed( FALSE );
+		if( QLocale::system().language() == QLocale::Hebrew ||
+			QLocale::system().language() == QLocale::Arabic )
+		{
+			QApplication::setLayoutDirection( Qt::RightToLeft );
+		}
 		app = a;
 	}
 
@@ -140,17 +141,17 @@ int ICAMain( int argc, char * * argv )
 	app->setEventFilter( eventFilter );
 #endif
 
-	const QString loc = QLocale::system().name().left( 2 );
-	QTranslator app_tr;
-	app_tr.load( ":/resources/" + loc + ".qm" );
-	app->installTranslator( &app_tr );
-
-	QTranslator qt_tr;
-	qt_tr.load( ":/resources/qt_" + loc + ".qm" );
-	app->installTranslator( &qt_tr );
-
 	localSystem::initialize();
 
+	if( localSystem::parameter( "ivsport" ).toInt() > 0 )
+	{
+		__ivs_port = localSystem::parameter( "ivsport" ).toInt();
+	}
+	if( localSystem::parameter( "isdport" ).toInt() > 0 )
+	{
+		__isd_port = localSystem::parameter( "isdport" ).toInt();
+	}
+
 	QStringListIterator arg_it( QCoreApplication::arguments() );
 	arg_it.next();
 	while( argc > 1 && arg_it.hasNext() )
@@ -175,18 +176,6 @@ int ICAMain( int argc, char * * argv )
 			return( isdServer::showAccessDialog( arg_it.next() ) );
 		}
 #endif
-		else if( a == "-rctrl" && arg_it.hasNext() )
-		{
-			const QString host = arg_it.next();
-			bool view_only = arg_it.hasNext() ?
-						arg_it.next().toInt()
-					:
-						FALSE;
-			new remoteControlWidget( host, view_only );
-			app->connect( app, SIGNAL( lastWindowClosed() ),
-							SLOT( quit() ) );
-			return( app->exec() );
-		}
 		else if( a == "-role" )
 		{
 			if( arg_it.hasNext() )
@@ -236,8 +225,8 @@ int ICAMain( int argc, char * * argv )
 			pkey.save( priv );
 			publicDSAKey( pkey ).save( pub );
 			printf( "...done, saved key-pair in\n\n%s\n\nand\n\n%s",
-						priv.toAscii().constData(),
-						pub.toAscii().constData() );
+						priv.toUtf8().constData(),
+						pub.toUtf8().constData() );
 			printf( "\n\n\nFor now the file is only readable by "
 				"root and members of group root (if you\n"
 				"didn't ran this command as non-root).\n"
@@ -260,10 +249,20 @@ int ICAMain( int argc, char * * argv )
 			return( 0 );
 		}
 #endif
+		else if( a == "-v" || a == "--version" )
+		{
+			printf( "%s\n", PACKAGE_VERSION );
+			return( 0 );
+		}
+		else if( a == "-l" || a == "--lock" )
+		{
+			lockWidget l;
+			return app->exec();
+		}
 		else
 		{
 			printf( "Unrecognized commandline-argument %s\n",
-						a.toAscii().constData() );
+						a.toUtf8().constData() );
 			return( -1 );
 		}
 	}
@@ -287,9 +286,11 @@ int ICAMain( int argc, char * * argv )
 	icon.addFile( ":/resources/icon22.png" );
 	icon.addFile( ":/resources/icon32.png" );
 
-	__systray_icon = new QSystemTrayIcon( icon );
+	QSystemTrayIcon sti( icon );
+	__systray_icon = &sti;
 	__systray_icon->setToolTip(
-				QApplication::tr( "iTALC Client on %1:%2" ).
+				QApplication::tr( "iTALC Client %1 on %2:%3" ).
+					arg( PACKAGE_VERSION ).
 					arg( QHostInfo::localHostName() ).
 					arg( QString::number( __ivs_port ) ) );
 	__systray_icon->show();
@@ -338,7 +339,7 @@ int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
 								++it, ++argc )
 	{
 		argv[argc] = new char[it->length() + 1];
-		strcpy( argv[argc], it->toAscii().constData() );
+		strcpy( argv[argc], it->toUtf8().constData() );
 	}
 
 	return( ICAMain( argc, argv ) );
