@@ -1,8 +1,8 @@
 /*
  * isd_server.cpp - ISD Server
  *
- * Copyright (c) 2006-2008 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
- *  
+ * Copyright (c) 2006-2010 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ *
  * This file is part of iTALC - http://italc.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
@@ -67,7 +67,14 @@ isdServer::isdServer( const quint16 _ivs_port, int _argc, char * * _argv ) :
 	m_readyReadMapper( this ),
 	m_ivs( NULL ),
 	m_demoClient( NULL ),
+#ifdef BUILD_WIN32
+	m_lockDesktop( NULL ),
+	m_origThreadDesktop( NULL ),
+	m_origInputDesktop( NULL ),
+	m_lockProcess( NULL )
+#else
 	m_lockWidget( NULL )
+#endif
 {
 	if( __isd_server ||
 			listen( QHostAddress::Any, __isd_port ) == FALSE )
@@ -111,14 +118,98 @@ isdServer::isdServer( const quint16 _ivs_port, int _argc, char * * _argv ) :
 
 isdServer::~isdServer()
 {
+	// this unlocks display and frees resources
+	unlockDisplay();
+
 	delete m_ivs;
-	delete m_lockWidget;
 	__isd_server = NULL;
 }
 
 
 
 
+#ifdef BUILD_WIN32
+static HANDLE runProcessAsLoggedOnUser( const QString & _cmd,
+								const QString & _desktop = "winsta0\\default" )
+{
+	// run process as the user which is logged on
+	DWORD aProcesses[1024], cbNeeded;
+
+	if( !EnumProcesses( aProcesses, sizeof( aProcesses ), &cbNeeded ) )
+	{
+		return NULL;
+	}
+
+	DWORD cProcesses = cbNeeded / sizeof(DWORD);
+
+	for( DWORD i = 0; i < cProcesses; i++ )
+	{
+		HANDLE hProcess = OpenProcess( PROCESS_ALL_ACCESS, false, aProcesses[i] );
+		HMODULE hMod;
+		if( hProcess == NULL ||
+			!EnumProcessModules( hProcess, &hMod, sizeof( hMod ), &cbNeeded ) )
+        {
+			continue;
+		}
+
+		TCHAR szProcessName[MAX_PATH];
+		GetModuleBaseName( hProcess, hMod, szProcessName, 
+                       		  sizeof( szProcessName ) / sizeof( TCHAR) );
+		for( TCHAR * ptr = szProcessName; *ptr; ++ptr )
+		{
+			*ptr = tolower( *ptr );
+		}
+
+		if( strcmp( szProcessName, "explorer.exe" ) )
+		{
+			CloseHandle( hProcess );
+			continue;
+		}
+	
+		HANDLE hToken;
+		OpenProcessToken( hProcess, MAXIMUM_ALLOWED, &hToken );
+		ImpersonateLoggedOnUser( hToken );
+
+		STARTUPINFO si;
+		PROCESS_INFORMATION pi;
+		ZeroMemory( &si, sizeof( STARTUPINFO ) );
+		si.cb= sizeof( STARTUPINFO );
+		si.lpDesktop = (CHAR *) qstrdup( _desktop.toUtf8().constData() );
+		HANDLE hNewToken = NULL;
+
+		DuplicateTokenEx( hToken, MAXIMUM_ALLOWED, NULL,
+					SecurityImpersonation, TokenPrimary,
+								&hNewToken );
+
+		CreateProcessAsUser(
+				hNewToken,            // client's access token
+				NULL,              // file to execute
+				(CHAR *)_cmd.toUtf8().constData(),     // command line
+				NULL,              // pointer to process SECURITY_ATTRIBUTES
+				NULL,              // pointer to thread SECURITY_ATTRIBUTES
+				FALSE,             // handles are not inheritable
+				NORMAL_PRIORITY_CLASS | CREATE_NEW_CONSOLE,   // creation flags
+				NULL,              // pointer to new environment block 
+				NULL,              // name of current directory 
+				&si,               // pointer to STARTUPINFO structure
+				&pi                // receives information about new process
+				);
+
+		delete[] si.lpDesktop;
+
+		CloseHandle( hNewToken );
+		RevertToSelf();
+		CloseHandle( hToken );
+		CloseHandle( hProcess );
+
+		return pi.hProcess;
+	}
+
+	return NULL;
+}
+#endif
+
+
 int isdServer::processClient( socketDispatcher _sd, void * _user )
 {
 	socketDevice sdev( _sd, _user );
@@ -159,80 +250,18 @@ int isdServer::processClient( socketDispatcher _sd, void * _user )
 			const QString cmds = msg_in.arg( "cmds" ).toString();
 			if( !cmds.isEmpty() )
 			{
+				foreach( const QString & cmd, cmds.split( '\n' ) )
+				{
 #ifdef BUILD_WIN32
-	// run process as the user which is logged on
-	DWORD aProcesses[1024], cbNeeded;
-
-	if( !EnumProcesses( aProcesses, sizeof( aProcesses ), &cbNeeded ) )
-	{
-		break;
-	}
-
-	DWORD cProcesses = cbNeeded / sizeof(DWORD);
-
-	for( DWORD i = 0; i < cProcesses; i++ )
-	{
-		HANDLE hProcess = OpenProcess( PROCESS_ALL_ACCESS,
-							false, aProcesses[i] );
-		HMODULE hMod;
-		if( hProcess == NULL ||
-			!EnumProcessModules( hProcess, &hMod, sizeof( hMod ),
-								&cbNeeded ) )
-	        {
-			continue;
-		}
-
-		TCHAR szProcessName[MAX_PATH];
-		GetModuleBaseName( hProcess, hMod, szProcessName, 
-                       		  sizeof( szProcessName ) / sizeof( TCHAR) );
-		for( TCHAR * ptr = szProcessName; *ptr; ++ptr )
-		{
-			*ptr = tolower( *ptr );
-		}
-
-		if( strcmp( szProcessName, "explorer.exe" ) )
-		{
-			CloseHandle( hProcess );
-			continue;
-		}
-	
-		HANDLE hToken;
-		OpenProcessToken( hProcess, MAXIMUM_ALLOWED, &hToken );
-		ImpersonateLoggedOnUser( hToken );
-
-		STARTUPINFO si;
-		PROCESS_INFORMATION pi;
-		ZeroMemory( &si, sizeof( STARTUPINFO ) );
-		si.cb= sizeof( STARTUPINFO );
-		si.lpDesktop = (CHAR *) "winsta0\\default";
-		HANDLE hNewToken = NULL;
-
-		DuplicateTokenEx( hToken, MAXIMUM_ALLOWED, NULL,
-					SecurityImpersonation, TokenPrimary,
-								&hNewToken );
-
-		CreateProcessAsUser(
-				hNewToken,            // client's access token
-				NULL,              // file to execute
-				(CHAR *)cmds.toAscii().constData(),     // command line
-				NULL,              // pointer to process SECURITY_ATTRIBUTES
-				NULL,              // pointer to thread SECURITY_ATTRIBUTES
-				FALSE,             // handles are not inheritable
-				NORMAL_PRIORITY_CLASS | CREATE_NEW_CONSOLE,   // creation flags
-				NULL,              // pointer to new environment block 
-				NULL,              // name of current directory 
-				&si,               // pointer to STARTUPINFO structure
-				&pi                // receives information about new process
-				); 
-		CloseHandle( hNewToken );
-		RevertToSelf();
-		CloseHandle( hToken );
-		CloseHandle( hProcess );
-		break;
-	}
+					HANDLE hProcess = runProcessAsLoggedOnUser( cmd );
+					if( hProcess )
+					{
+						CloseHandle( hProcess );
+					}
 #else
-				QProcess::startDetached( cmds );
+					QProcess::startDetached( cmd );
 #endif
+				}
 			}
 			break;
 		}
@@ -788,6 +817,12 @@ void isdServer::checkForPendingActions( void )
 		}
 		m_pendingActions.removeFirst();
 	}
+#ifdef BUILD_WIN32
+	if( m_lockDesktop )
+	{
+		SwitchDesktop( m_lockDesktop );
+	}
+#endif
 }
 
 
@@ -829,24 +864,49 @@ void isdServer::stopDemo( void )
 
 
 
-
 void isdServer::lockDisplay( void )
 {
 	if( demoServer::numOfInstances() )
 	{
 		return;
 	}
+
+#ifdef BUILD_WIN32
+	m_origThreadDesktop = GetThreadDesktop( GetCurrentThreadId() );
+	m_origInputDesktop = OpenInputDesktop( 0, FALSE, DESKTOP_SWITCHDESKTOP );
+
+	char desktopName[] = "LockDesktop";
+	m_lockDesktop = CreateDesktop( desktopName, NULL, NULL, 0, GENERIC_ALL, NULL );
+	SetThreadDesktop( m_lockDesktop );
+
+	m_lockProcess =
+		runProcessAsLoggedOnUser(
+					QCoreApplication::applicationFilePath().
+							replace( '/', QDir::separator() ) + " --lock",
+					desktopName );
+
+	SwitchDesktop( m_lockDesktop );
+#else
 	delete m_lockWidget;
 	m_lockWidget = new lockWidget();
+#endif
 }
 
 
 
 
-void isdServer::unlockDisplay( void )
+void isdServer::unlockDisplay()
 {
+#ifdef BUILD_WIN32
+	SwitchDesktop( m_origInputDesktop );
+	SetThreadDesktop( m_origThreadDesktop );
+
+	TerminateProcess( m_lockProcess, 0 );
+	CloseDesktop( m_lockDesktop );
+#else
 	delete m_lockWidget;
 	m_lockWidget = NULL;
+#endif
 }
 
 
