@@ -60,7 +60,7 @@ qint64 libvncClientDispatcher( char * _buf, const qint64 _len,
 		case SocketWrite:
 			return( rfbWriteExact( cl, _buf, _len ) == 1 ? _len :
 									0 );
-		case SocketGetIPBoundTo:
+		case SocketGetPeerAddress:
 			strncpy( _buf, cl->host, _len );
 			break;
 	}
@@ -177,6 +177,7 @@ void IVS::run( void )
 		while( 1 )
 		{
 			QProcess p;
+			p.setProcessChannelMode( QProcess::ForwardedChannels );
 			p.start( QCoreApplication::applicationFilePath() +
 					" -rx11vs " + cmdline.join( " " ) );
 			m_restart = FALSE;
@@ -194,6 +195,9 @@ void IVS::run( void )
 		}
 		return;
 	}
+#if 0
+	cmdline	<< "-dbg";// << "-o" << "/tmp/italc_client_x11vnc.log";
+#endif
 
 	char * old_av = m_argv[0];
 	m_argv = new char *[cmdline.size()+1];
