@@ -1,7 +1,7 @@
 /*
  * system_service.cpp - implementation of systemService-class
  *
- * Copyright (c) 2006 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ * Copyright (c) 2006-2010 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
  *  
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -40,6 +40,12 @@
 #endif
 
 
+#ifdef BUILD_WIN32
+PFN_WTSQuerySessionInformation pfnWTSQuerySessionInformation = NULL;
+PFN_WTSFreeMemory pfnWTSFreeMemory = NULL;
+#endif
+
+
 systemService::systemService(
 			const QString & _service_name,
 			const QString & _service_arg,
@@ -151,8 +157,8 @@ bool systemService::install( void )
 #ifdef NO_GUI
 	if( isDebian() )
 	{
-		QProcess::execute( QString( "update-rc.d %1 defaults 50" ).
-								arg( m_name ) );
+		return( QProcess::execute( QString( "update-rc.d %1 defaults 50" ).
+								arg( m_name ) ) == 0 );
 	}
 #else
 	if( isDebian() )
@@ -166,8 +172,8 @@ bool systemService::install( void )
 			QFile f( "/etc/kde3/kdm/Xsetup" );
 			f.open( QFile::WriteOnly );
 			f.seek( f.size() );
-			f.write( QString( "%1 %2" ).arg( sn ).arg( "start" ).
-								toUtf8() );
+			return( f.write( QString( "%1 %2" ).arg( sn ).arg( "start" ).
+								toUtf8() ) > 0 );
 		}
 		else if( dm == "gdm" )
 		{
@@ -180,12 +186,14 @@ bool systemService::install( void )
 		}
 	}
 #endif
+	return( FALSE );
 }
 
 
 bool systemService::remove( void )
 {
 	// TODO
+	return( FALSE );
 }
 
 
@@ -215,7 +223,6 @@ bool systemService::runAsService( void )
 	{
 		return( FALSE );
 	}
-	//new workThread( serviceMainThread );
 	m_serviceMain( this );
 	m_running = FALSE;
 	return( TRUE );
@@ -271,7 +278,7 @@ bool systemService::install( void )
 	if( strlen( path ) + 4 + m_arg.length() < pathlength )
 	{
 		sprintf( servicecmd, "\"%s\" %s", path,
-						m_arg.toUtf8().constData() );
+					m_arg.toLocal8Bit().constData() );
 	}
 	else
 	{
@@ -298,8 +305,8 @@ bool systemService::install( void )
 	// Create an entry for the WinVNC service
 	SC_HANDLE hservice = CreateService(
 			hsrvmanager,		// SCManager database
-			m_name.toUtf8().constData(),	// name of service
-			m_displayName.toUtf8().constData(),// name to display
+			m_name.toLocal8Bit().constData(),	// name of service
+			m_displayName.toLocal8Bit().constData(),// name to display
 			SERVICE_ALL_ACCESS,	// desired access
 			SERVICE_WIN32_OWN_PROCESS | SERVICE_INTERACTIVE_PROCESS,
 						// service type
@@ -308,7 +315,7 @@ bool systemService::install( void )
 			servicecmd,		// service's binary
 			NULL,			// no load ordering group
 			NULL,			// no tag identifier
-			m_dependencies.toUtf8().constData(), // dependencies
+			NULL,//m_dependencies.toLocal8Bit().constData(), // dependencies
 			NULL,			// LocalSystem account
 			NULL );			// no password
 	if( hservice == NULL)
@@ -387,7 +394,7 @@ bool systemService::remove( void )
 	if( hsrvmanager )
 	{ 
 		SC_HANDLE hservice = OpenService( hsrvmanager,
-						m_name.toUtf8().constData(),
+						m_name.toLocal8Bit().constData(),
 							SERVICE_ALL_ACCESS );
 
 		if( hservice != NULL )
@@ -582,7 +589,7 @@ bool systemService::stop( void )
 	if( hsrvmanager )
 	{ 
 		SC_HANDLE hservice = OpenService( hsrvmanager,
-						m_name.toUtf8().constData(),
+					m_name.toLocal8Bit().constData(),
 							SERVICE_ALL_ACCESS );
 
 		if( hservice != NULL )
@@ -662,7 +669,7 @@ bool systemService::runAsService( void )
 	// Create a service entry table
 	SERVICE_TABLE_ENTRY dispatchTable[] =
 	{
-		{ m_name.toUtf8().data(), (LPSERVICE_MAIN_FUNCTION) main },
+		{ m_name.toLocal8Bit().data(), (LPSERVICE_MAIN_FUNCTION) main },
 		{ NULL, NULL }
 	} ;
 
@@ -711,10 +718,19 @@ void systemService::serviceMainThread( void * _arg )
 
 void WINAPI systemService::main( DWORD _argc, char * * _argv )
 {
+	HMODULE hWTSAPI32 = LoadLibrary("wtsapi32.dll");
+	if( hWTSAPI32 )
+	{
+		pfnWTSQuerySessionInformation = (PFN_WTSQuerySessionInformation) 
+			GetProcAddress(hWTSAPI32,"WTSQuerySessionInformationA");
+		pfnWTSFreeMemory = (PFN_WTSFreeMemory) 
+      		GetProcAddress(hWTSAPI32,"WTSFreeMemory");
+	}
+	DWORD context = 1;
 	// register the service control handler
-	s_statusHandle = RegisterServiceCtrlHandler(
-					s_this->m_name.toUtf8().constData(),
-								serviceCtrl );
+	s_statusHandle = RegisterServiceCtrlHandlerEx(
+					s_this->m_name.toLocal8Bit().constData(),
+								serviceCtrl, &context );
 
 	if( s_statusHandle == 0 )
 	{
@@ -723,7 +739,11 @@ void WINAPI systemService::main( DWORD _argc, char * * _argv )
 
 	// Set up some standard service state values
 	s_status.dwServiceType = SERVICE_WIN32 | SERVICE_INTERACTIVE_PROCESS;
-	s_status.dwServiceSpecificExitCode = 0;
+	s_status.dwServiceSpecificExitCode = NO_ERROR;
+	s_status.dwControlsAccepted = 0;
+	s_status.dwWin32ExitCode = NO_ERROR;
+	s_status.dwCheckPoint = 0;
+	s_status.dwWaitHint = 0;
 
 	// Give this status to the SCM
 	if( !reportStatus(
@@ -737,17 +757,18 @@ void WINAPI systemService::main( DWORD _argc, char * * _argv )
 		return;
 	}
 
-	//serviceMainThread(0);
-	// now start the service for real
-	//omni_thread::create( serviceMainThread, s_this );
-	( new workThread( serviceMainThread, s_this ) )->start();
-	return;
+	serviceMainThread( s_this );
 }
 
 
 
+
+QString __sessCurUser;
+
 // Service control routine
-void WINAPI systemService::serviceCtrl( DWORD _ctrlcode )
+DWORD WINAPI systemService::serviceCtrl( DWORD _ctrlcode, DWORD dwEventType,
+							LPVOID lpEventData,
+							LPVOID lpContext )
 {
 	// What control code have we been sent?
 	switch( _ctrlcode )
@@ -766,6 +787,33 @@ void WINAPI systemService::serviceCtrl( DWORD _ctrlcode )
 		case SERVICE_CONTROL_INTERROGATE:
 			// Service control manager just wants to know our state
 			break;
+			
+		case SERVICE_CONTROL_SESSIONCHANGE:
+			WTSSESSION_NOTIFICATION wtsno;
+			CopyMemory( &wtsno, lpEventData,
+					sizeof( WTSSESSION_NOTIFICATION ) );
+			
+			switch( dwEventType )
+			{
+				case WTS_SESSION_LOGOFF:
+					__sessCurUser = "";
+					break;
+				case WTS_SESSION_LOGON:
+					LPTSTR pBuffer = NULL;
+					DWORD dwBufferLen;
+	BOOL bRes = (*pfnWTSQuerySessionInformation)( WTS_CURRENT_SERVER_HANDLE,
+							wtsno.dwSessionId,
+							WTSUserName,
+							&pBuffer,
+							&dwBufferLen );
+					if( bRes != FALSE )
+					{
+						__sessCurUser = pBuffer;
+					}
+					(*pfnWTSFreeMemory)( pBuffer );
+					break;
+			}
+			break;
 
 		default:
 			// Control code not recognised
@@ -774,6 +822,7 @@ void WINAPI systemService::serviceCtrl( DWORD _ctrlcode )
 
 	// Tell the control manager what we're up to.
 	reportStatus( s_status.dwCurrentState, NO_ERROR, 0 );
+	return NO_ERROR;
 }
 
 
@@ -793,7 +842,14 @@ bool systemService::reportStatus( DWORD _state, DWORD _exit_code,
 	}
 	else
 	{
-		s_status.dwControlsAccepted = SERVICE_ACCEPT_STOP;
+		if( QSysInfo::WindowsVersion == QSysInfo::WV_2000 )
+		{
+			s_status.dwControlsAccepted = SERVICE_ACCEPT_STOP;
+		}
+		else
+		{
+			s_status.dwControlsAccepted = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SESSIONCHANGE;
+		}
 	}
 
 	// Save the new status we've been given
