@@ -25,7 +25,10 @@
 
 
 // #define VC_EXTRALEAN
+#ifndef STRICT
 #define STRICT
+#endif
+#define WIN32_LEAN_AND_MEAN
 
 #include <winsock2.h>
 #include <windows.h>
