@@ -844,6 +844,8 @@ vncClientThread::run(void *arg)
 				break;
 			}
 
+			vnclog.Print(LL_INTINFO, VNCLOG("SetPixelFormat message received\n"));
+
 			// Swap the relevant bits.
 			msg.spf.format.redMax = Swap16IfLE(msg.spf.format.redMax);
 			msg.spf.format.greenMax = Swap16IfLE(msg.spf.format.greenMax);
@@ -873,6 +875,8 @@ vncClientThread::run(void *arg)
 				break;
 			}
 
+			vnclog.Print(LL_INTINFO, VNCLOG("SetEncodings message received\n"));
+
 			m_client->m_buffer->SetQualityLevel(-1);
 			m_client->m_buffer->SetCompressLevel(6);
 			m_client->m_buffer->EnableXCursor(FALSE);
@@ -1025,6 +1029,12 @@ vncClientThread::run(void *arg)
 				break;
 			}
 
+			if (msg.fur.incremental) {
+				vnclog.Print(LL_INTINFO, VNCLOG("FramebufferUpdateRequest(incr) received\n"));
+			} else {
+				vnclog.Print(LL_INTINFO, VNCLOG("FramebufferUpdateRequest(full) received\n"));
+			}
+
 			{
 				RECT update;
 				RECT sharedRect;
@@ -1087,6 +1097,8 @@ vncClientThread::run(void *arg)
 			// Read the rest of the message:
 			if (m_socket->ReadExact(((char *) &msg)+1, sz_rfbKeyEventMsg-1))
 			{
+				vnclog.Print(LL_INTINFO, VNCLOG("KeyEvent message received\n"));
+
 				if (m_client->IsKeyboardEnabled() && !m_client->IsInputBlocked())
 				{
 					msg.ke.key = Swap32IfLE(msg.ke.key);
@@ -1102,6 +1114,8 @@ vncClientThread::run(void *arg)
 			// Read the rest of the message:
 			if (m_socket->ReadExact(((char *) &msg)+1, sz_rfbPointerEventMsg-1))
 			{
+				vnclog.Print(LL_INTINFO, VNCLOG("PointerEvent message received\n"));
+
 				if (m_client->IsPointerEnabled() && !m_client->IsInputBlocked())
 				{
 					// Convert the coords to Big Endian
@@ -1206,6 +1220,8 @@ vncClientThread::run(void *arg)
 			// Read the rest of the message:
 			if (m_socket->ReadExact(((char *) &msg)+1, sz_rfbClientCutTextMsg-1))
 			{
+				vnclog.Print(LL_INTINFO, VNCLOG("ClientCutText message received\n"));
+
 				// Allocate storage for the text
 				const UINT length = Swap32IfLE(msg.cct.length);
 				char *text = new char [length+1];
@@ -1236,17 +1252,40 @@ vncClientThread::run(void *arg)
 			}
 			if (m_socket->ReadExact(((char *) &msg)+1, sz_rfbFileListRequestMsg-1))
 			{
+				vnclog.Print(LL_INTINFO, VNCLOG("FileListRequest message received\n"));
+
 				msg.flr.dirNameSize = Swap16IfLE(msg.flr.dirNameSize);
 				if (msg.flr.dirNameSize > 255) break;
 				char path[255 + 1];
 				m_socket->ReadExact(path, msg.flr.dirNameSize);
 				path[msg.flr.dirNameSize] = '\0';
 				ConvertPath(path);
+				if (!vncService::tryImpersonate()) {
+					omni_mutex_lock l(m_client->m_sendUpdateLock);
+					rfbFileListDataMsg fld;
+					fld.type = rfbFileListData;
+					fld.numFiles = Swap16IfLE(0);
+					fld.dataSize = Swap16IfLE(0);
+					fld.compressedSize = Swap16IfLE(0);
+					fld.flags = msg.flr.flags | 0x80;
+					m_socket->SendExact((char *)&fld, sz_rfbFileListDataMsg);
+					break;
+				}
 				FileTransferItemInfo ftii;
 				if (strlen(path) == 0) {
 					TCHAR szDrivesList[256];
-					if (GetLogicalDriveStrings(255, szDrivesList) == 0)
+					if (GetLogicalDriveStrings(255, szDrivesList) == 0) {
+						omni_mutex_lock l(m_client->m_sendUpdateLock);
+						rfbFileListDataMsg fld;
+						fld.type = rfbFileListData;
+						fld.numFiles = Swap16IfLE(0);
+						fld.dataSize = Swap16IfLE(0);
+						fld.compressedSize = Swap16IfLE(0);
+						fld.flags = msg.flr.flags | 0x80;
+						m_socket->SendExact((char *)&fld, sz_rfbFileListDataMsg);
+						vncService::undoImpersonate();
 						break;
+					}
 					int i = 0;
 					while (szDrivesList[i] != '\0') {
 						char *drive = strdup(&szDrivesList[i]);
@@ -1261,10 +1300,10 @@ vncClientThread::run(void *arg)
 					strcat(path, "\\*");
 					HANDLE FLRhandle;
 					WIN32_FIND_DATA FindFileData;
-					SetErrorMode(SEM_FAILCRITICALERRORS);
+					UINT savedErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
 					FLRhandle = FindFirstFile(path, &FindFileData);
 					DWORD LastError = GetLastError();
-					SetErrorMode(0);
+					SetErrorMode(savedErrorMode);
 					if (FLRhandle != INVALID_HANDLE_VALUE) {
 						do {
 							if (strcmp(FindFileData.cFileName, ".") != 0 &&
@@ -1282,6 +1321,7 @@ vncClientThread::run(void *arg)
 							}
 
 						} while (FindNextFile(FLRhandle, &FindFileData));
+						FindClose(FLRhandle);
 					} else {
 						if (LastError != ERROR_SUCCESS && LastError != ERROR_FILE_NOT_FOUND) {
 							omni_mutex_lock l(m_client->m_sendUpdateLock);
@@ -1293,10 +1333,10 @@ vncClientThread::run(void *arg)
 							fld.compressedSize = Swap16IfLE(0);
 							fld.flags = msg.flr.flags | 0x80;
 							m_socket->SendExact((char *)&fld, sz_rfbFileListDataMsg);
+							vncService::undoImpersonate();
 							break;
 						}
 					}
-					FindClose(FLRhandle);	
 				}
 				int dsSize = ftii.GetNumEntries() * 8;
 				int msgLen = sz_rfbFileListDataMsg + dsSize + ftii.GetSummaryNamesLength() + ftii.GetNumEntries();
@@ -1317,6 +1357,7 @@ vncClientThread::run(void *arg)
 				}
 				omni_mutex_lock l(m_client->m_sendUpdateLock);
 				m_socket->SendExact(pAllMessage, msgLen);
+				vncService::undoImpersonate();
 			}
 			break;
 
@@ -1327,16 +1368,27 @@ vncClientThread::run(void *arg)
 			}
 			if (m_socket->ReadExact(((char *) &msg)+1, sz_rfbFileDownloadRequestMsg-1))
 			{
+				vnclog.Print(LL_INTINFO, VNCLOG("FileDownloadRequest message received\n"));
+
 				msg.fdr.fNameSize = Swap16IfLE(msg.fdr.fNameSize);
 				msg.fdr.position = Swap32IfLE(msg.fdr.position);
+
+				if (!vncService::tryImpersonate()) {
+					m_socket->ReadExact(NULL, msg.fdr.fNameSize);
+					char reason[] = "Cannot impersonate logged on user";
+					int reasonLen = strlen(reason);
+					m_client->SendFileDownloadFailed(reasonLen, reason);
+					break;
+				}
 				if (msg.fdr.fNameSize > 255) {
 					m_socket->ReadExact(NULL, msg.fdr.fNameSize);
 					char reason[] = "Path length exceeds 255 bytes";
 					int reasonLen = strlen(reason);
 					m_client->SendFileDownloadFailed(reasonLen, reason);
+					vncService::undoImpersonate();
 					break;
 				}
-				char path_file[255];
+				char path_file[255 + 1];
 				m_socket->ReadExact(path_file, msg.fdr.fNameSize);
 				path_file[msg.fdr.fNameSize] = '\0';
 				ConvertPath(path_file);
@@ -1348,10 +1400,10 @@ vncClientThread::run(void *arg)
 				DWORD dwNumberOfBytesRead = 0;
 				DWORD dwNumberOfAllBytesRead = 0;
 				WIN32_FIND_DATA FindFileData;
-				SetErrorMode(SEM_FAILCRITICALERRORS);
+				UINT savedErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
 				hFile = FindFirstFile(path_file, &FindFileData);
 				DWORD LastError = GetLastError();
-				SetErrorMode(0);
+				SetErrorMode(savedErrorMode);
 
 				vnclog.Print(LL_CLIENTS, VNCLOG("file download requested: %s\n"),
 							 path_file);
@@ -1362,6 +1414,7 @@ vncClientThread::run(void *arg)
 					char reason[] = "Cannot open file, perhaps it is absent or is a directory";
 					int reasonLen = strlen(reason);
 					m_client->SendFileDownloadFailed(reasonLen, reason);
+					vncService::undoImpersonate();
 					break;
 				}
 				sz_rfbFileSize = FindFileData.nFileSizeLow;
@@ -1371,14 +1424,15 @@ vncClientThread::run(void *arg)
 					m_client->SendFileDownloadData(m_client->m_modTime);
 				} else {
 					if (sz_rfbFileSize <= sz_rfbBlockSize) sz_rfbBlockSize = sz_rfbFileSize;
-					SetErrorMode(SEM_FAILCRITICALERRORS);
+					UINT savedErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
 					m_client->m_hFileToRead = CreateFile(path_file, GENERIC_READ, FILE_SHARE_READ, NULL,	OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
-					SetErrorMode(0);
+					SetErrorMode(savedErrorMode);
 					if (m_client->m_hFileToRead != INVALID_HANDLE_VALUE) {
 						m_client->m_bDownloadStarted = TRUE;
 						m_client->SendFileDownloadPortion();
 					}
 				}
+				vncService::undoImpersonate();
 			}
 			break;
 
@@ -1389,13 +1443,24 @@ vncClientThread::run(void *arg)
 			}
 			if (m_socket->ReadExact(((char *) &msg)+1, sz_rfbFileUploadRequestMsg-1))
 			{
+				vnclog.Print(LL_INTINFO, VNCLOG("FileUploadRequest message received\n"));
+
 				msg.fupr.fNameSize = Swap16IfLE(msg.fupr.fNameSize);
 				msg.fupr.position = Swap32IfLE(msg.fupr.position);
+
+				if (!vncService::tryImpersonate()) {
+					m_socket->ReadExact(NULL, msg.fupr.fNameSize);
+					char reason[] = "Cannot impersonate logged on user";
+					int reasonLen = strlen(reason);
+					m_client->SendFileUploadCancel(reasonLen, reason);
+					break;
+				}
 				if (msg.fupr.fNameSize > MAX_PATH) {
 					m_socket->ReadExact(NULL, msg.fupr.fNameSize);
 					char reason[] = "Path length exceeds MAX_PATH value";
 					int reasonLen = strlen(reason);
 					m_client->SendFileUploadCancel(reasonLen, reason);
+					vncService::undoImpersonate();
 					break;
 				}
 				m_socket->ReadExact(m_client->m_UploadFilename, msg.fupr.fNameSize);
@@ -1409,9 +1474,11 @@ vncClientThread::run(void *arg)
 					char reason[] = "Could not create file";
 					int reasonLen = strlen(reason);
 					m_client->SendFileUploadCancel(reasonLen, reason);
+					vncService::undoImpersonate();
+					break;
 				}
-				DWORD dwError = GetLastError();
 				/*
+				DWORD dwError = GetLastError();
 				SYSTEMTIME systime;
 				FILETIME filetime;
 				GetSystemTime(&systime);
@@ -1431,6 +1498,7 @@ vncClientThread::run(void *arg)
 					}
 				}
 				*/
+				vncService::undoImpersonate();
 			}				
 			break;
 
@@ -1441,8 +1509,23 @@ vncClientThread::run(void *arg)
 			}
 			if (m_socket->ReadExact(((char *) &msg)+1, sz_rfbFileUploadDataMsg-1))
 			{
+				vnclog.Print(LL_INTINFO, VNCLOG("FileUploadData message received\n"));
+
 				msg.fud.realSize = Swap16IfLE(msg.fud.realSize);
 				msg.fud.compressedSize = Swap16IfLE(msg.fud.compressedSize);
+
+				if (!vncService::tryImpersonate()) {
+					if (msg.fud.realSize == 0 && msg.fud.compressedSize == 0) {
+						m_socket->ReadExact(NULL, sizeof(CARD32));
+					} else {
+						m_socket->ReadExact(NULL, msg.fud.compressedSize);
+					}
+					char reason[] = "Cannot impersonate logged on user";
+					int reasonLen = strlen(reason);
+					m_client->SendFileUploadCancel(reasonLen, reason);
+					m_client->CloseUndoneFileTransfer();
+					break;
+				}
 				if ((msg.fud.realSize == 0) && (msg.fud.compressedSize == 0)) {
 					CARD32 mTime;
 					m_socket->ReadExact((char *) &mTime, sizeof(CARD32));
@@ -1466,27 +1549,31 @@ vncClientThread::run(void *arg)
 //								 m_client->m_UploadFilename, dwBytePerSecond, dwFileSize, uploadTime);
 					vnclog.Print(LL_CLIENTS, VNCLOG("file upload complete: %s;\n"),
 								 m_client->m_UploadFilename);
+					vncService::undoImpersonate();
 					break;
 				}
 				DWORD dwNumberOfBytesWritten;
 				char *pBuff = new char [msg.fud.compressedSize];
 				m_socket->ReadExact(pBuff, msg.fud.compressedSize);
 				if (msg.fud.compressedLevel != 0) {
+					delete[] pBuff;
 					char reason[] = "Server does not support data compression on upload";
 					int reasonLen = strlen(reason);
 					m_client->SendFileUploadCancel(reasonLen, reason);
 					m_client->CloseUndoneFileTransfer();
+					vncService::undoImpersonate();
 					break;
 				}
 				BOOL bResult = WriteFile(m_client->m_hFileToWrite, pBuff, msg.fud.compressedSize, &dwNumberOfBytesWritten, NULL);
+				delete[] pBuff;
 				if ((dwNumberOfBytesWritten != msg.fud.compressedSize) || !bResult) {
 					char reason[] = "Error writing file data";
 					int reasonLen = strlen(reason);
 					m_client->SendFileUploadCancel(reasonLen, reason);
 					m_client->CloseUndoneFileTransfer();
+					vncService::undoImpersonate();
 					break;
 				}
-				delete [] pBuff;
 			}
 			break;
 
@@ -1497,12 +1584,16 @@ vncClientThread::run(void *arg)
 			}
 			if (m_socket->ReadExact(((char *) &msg)+1, sz_rfbFileDownloadCancelMsg-1))
 			{
+				vnclog.Print(LL_INTINFO, VNCLOG("FileDownloadCancel message received\n"));
+
+				vncService::tryImpersonate();
 				msg.fdc.reasonLen = Swap16IfLE(msg.fdc.reasonLen);
 				char *reason = new char[msg.fdc.reasonLen + 1];
 				m_socket->ReadExact(reason, msg.fdc.reasonLen);
 				reason[msg.fdc.reasonLen] = '\0';
 				m_client->CloseUndoneFileTransfer();
 				delete [] reason;
+				vncService::undoImpersonate();
 			}
 			break;
 
@@ -1513,12 +1604,16 @@ vncClientThread::run(void *arg)
 			}
 			if (m_socket->ReadExact(((char *) &msg)+1, sz_rfbFileUploadFailedMsg-1))
 			{
+				vnclog.Print(LL_INTINFO, VNCLOG("FileUploadFailed message received\n"));
+
+				vncService::tryImpersonate();
 				msg.fuf.reasonLen = Swap16IfLE(msg.fuf.reasonLen);
 				char *reason = new char[msg.fuf.reasonLen + 1];
 				m_socket->ReadExact(reason, msg.fuf.reasonLen);
 				reason[msg.fuf.reasonLen] = '\0';
 				m_client->CloseUndoneFileTransfer();
 				delete [] reason;
+				vncService::undoImpersonate();
 			}
 			break;
 
@@ -1529,6 +1624,9 @@ vncClientThread::run(void *arg)
 			}
 			if (m_socket->ReadExact(((char *) &msg)+1, sz_rfbFileCreateDirRequestMsg-1))
 			{
+				vnclog.Print(LL_INTINFO, VNCLOG("FileCreateDirRequest message received\n"));
+
+				vncService::tryImpersonate();
 				msg.fcdr.dNameLen = Swap16IfLE(msg.fcdr.dNameLen);
 				char *dirName = new char[msg.fcdr.dNameLen + 1];
 				m_socket->ReadExact(dirName, msg.fcdr.dNameLen);
@@ -1536,16 +1634,20 @@ vncClientThread::run(void *arg)
 				dirName = ConvertPath(dirName);
 				CreateDirectory((LPCTSTR) dirName, NULL);
 				delete [] dirName;
+				vncService::undoImpersonate();
 			}
 
 			break;
 #endif
 		case rfbItalcServiceRequest:
+		{
+			omni_mutex_lock l(m_client->m_sendUpdateLock);
 			if( !processItalcClient( vsocketDispatcher, m_socket ) )
 			{
 				connected = FALSE;
 			}
 			break;
+		}
 
 		default:
 			// Unknown message, so fail!
@@ -1799,9 +1901,9 @@ vncClient::UpdateRegion(vncRegion &region)
 void
 vncClient::CopyRect(RECT &dest, POINT &source)
 {
-	// If copyrect is disabled then just redraw the region!
-	if (!m_copyrect_use)
-	{
+	// If CopyRect encoding is disabled or we already have a CopyRect pending,
+	// then just redraw the region.
+	if (!m_copyrect_use || m_copyrect_set) {
 		UpdateRect(dest);
 		return;
 	}
@@ -1821,21 +1923,8 @@ vncClient::CopyRect(RECT &dest, POINT &source)
 
 		// Work out the source rectangle
 		RECT srcrect;
-
-		// Is this a continuation of an earlier window drag?
-		if (m_copyrect_set &&
-			((source.x == m_copyrect_rect.left) && (source.y == m_copyrect_rect.top)))
-		{
-			// Yes, so use the old source position
-			srcrect.left = m_copyrect_src.x;
-			srcrect.top = m_copyrect_src.y;
-		}
-		else
-		{
-			// No, so use this source position
-			srcrect.left = source.x;
-			srcrect.top = source.y;
-		}
+		srcrect.left = source.x;
+		srcrect.top = source.y;
 
 		// And fill out the right & bottom using the dest rect
 		srcrect.right = destrect.right-destrect.left + srcrect.left;
@@ -1852,15 +1941,6 @@ vncClient::CopyRect(RECT &dest, POINT &source)
 		destrect.right = srcrect2.right-srcrect2.left + destrect.left;
 		destrect.bottom = srcrect2.bottom-srcrect2.top + destrect.top;
 
-		// Is there an existing CopyRect rectangle?
-		if (m_copyrect_set)
-		{
-			// Yes, so compare their areas!
-			if (((destrect.right-destrect.left) * (destrect.bottom-destrect.top))
-				< ((m_copyrect_rect.right-m_copyrect_rect.left) * (m_copyrect_rect.bottom-m_copyrect_rect.top)))
-				return;
-		}
-
 		// Set the copyrect...
 		m_copyrect_rect = destrect;
 		m_copyrect_src.x = srcrect2.left;
