@@ -1278,7 +1278,7 @@ vncDesktop::InitWindow()
 	}
 
 	// Set the "this" pointer for the window
-	SetWindowLong(m_hwnd, GWL_USERDATA, (long)this);
+	SetWindowLongPtr(m_hwnd, GWLP_USERDATA, (LONG_PTR)this);
 
 	// Enable clipboard hooking
 	m_hnextviewer = SetClipboardViewer(m_hwnd);
@@ -1401,6 +1401,31 @@ void vncDesktop::CaptureScreen(const RECT &UpdateArea, BYTE *scrBuff)
 	else	CaptureScreenFromAdapterGeneral(UpdateArea, scrBuff);
 }
 
+
+static bool __hasLayeredWindows = false;
+
+static BOOL CALLBACK checkForLayeredWindow( HWND hwnd, LPARAM lParam )
+{
+	if( ( GetWindowLongPtr( hwnd, GWL_EXSTYLE ) & WS_EX_LAYERED ) &&
+			IsWindowVisible( hwnd ) &&
+			!IsIconic( hwnd ) )
+	{
+		__hasLayeredWindows = true;
+		return false;
+	}
+	return true;
+}
+
+
+static bool useCaptureBlt()
+{
+	__hasLayeredWindows = false;
+	EnumWindows((WNDENUMPROC)checkForLayeredWindow, 0);
+	return __hasLayeredWindows;
+}
+
+
+
 void vncDesktop::CaptureScreenFromAdapterGeneral(RECT rect, BYTE *scrBuff)
 {
 // ASSUME rect related to virtual desktop
@@ -1428,7 +1453,7 @@ void vncDesktop::CaptureScreenFromAdapterGeneral(RECT rect, BYTE *scrBuff)
 		rect.bottom - rect.top,
 		m_hrootdc,
 		rect.left, rect.top,
-		SRCCOPY);
+		useCaptureBlt() ? ( SRCCOPY | CAPTUREBLT ) : SRCCOPY );
 
 	// Select the old bitmap back into the memory DC
 	SelectObject(m_hmemdc, oldbitmap);
@@ -1781,7 +1806,7 @@ EnumWindowsFnCopyRect(HWND hwnd, LPARAM arg)
 {
 
 	//For excluding the popup windows
-	if ((GetWindowLong( hwnd, GWL_STYLE) & WS_POPUP) ==0)
+	if ((GetWindowLongPtr( hwnd, GWL_STYLE) & WS_POPUP) ==0)
 	{
 	
 		HANDLE prop = GetProp(hwnd, (LPCTSTR) MAKELONG(VNC_WINDOWPOS_ATOM, 0));
@@ -1873,7 +1898,7 @@ vncDesktop::CalcCopyRects()
 LRESULT CALLBACK
 DesktopWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
 {
-	vncDesktop *_this = (vncDesktop*)GetWindowLong(hwnd, GWL_USERDATA);
+	vncDesktop *_this = (vncDesktop*)GetWindowLongPtr(hwnd, GWLP_USERDATA);
 
 	switch (iMsg)
 	{
@@ -3084,7 +3109,7 @@ bool	bDbgBmDump(
 		"%04u.%02u.%02u-%02u-%02u-%02u-0x%08x.bmp",
 		stm.wYear, stm.wMonth, stm.wDay,
 		stm.wHour, stm.wMinute, stm.wSecond,
-		(uint) ptr);
+		(uint)(intptr_t) ptr);
 
 	HANDLE hFile = CreateFile(
 		szFileName,
