@@ -1,8 +1,17 @@
-#define _XOPEN_SOURCE
-
 #include <rfb/rfbconfig.h>
 #include <rfb/keysym.h>
 
+#ifdef LIBVNCSERVER_HAVE_RECORD
+#include <X11/Xproto.h>
+#endif
+
+#include "libvncclient/cursor.c"
+#include "libvncclient/tls.c"
+#include "libvncclient/listen.c"
+#include "libvncclient/rfbproto.c"
+#include "libvncclient/sockets.c"
+#include "libvncclient/vncviewer.c"
+
 #include "x11vnc/avahi.c"
 #include "x11vnc/pm.c"
 #include "x11vnc/rates.c"
