@@ -1,3 +1,35 @@
+/*
+   Copyright (C) 2002-2010 Karl J. Runge <runge@karlrunge.com> 
+   All rights reserved.
+
+This file is part of x11vnc.
+
+x11vnc is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or (at
+your option) any later version.
+
+x11vnc is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with x11vnc; if not, write to the Free Software
+Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
+or see <http://www.gnu.org/licenses/>.
+
+In addition, as a special exception, Karl J. Runge
+gives permission to link the code of its release of x11vnc with the
+OpenSSL project's "OpenSSL" library (or with modified versions of it
+that use the same license as the "OpenSSL" library), and distribute
+the linked executables.  You must obey the GNU General Public License
+in all respects for all of the code used other than "OpenSSL".  If you
+modify this file, you may extend this exception to your version of the
+file, but you are not obligated to do so.  If you do not wish to do
+so, delete this exception statement from your version.
+*/
+
 /* -- uinput.c -- */
 
 #include "x11vnc.h"
@@ -82,10 +114,10 @@ static char *devs[] = {
  */
 
 int check_uinput(void) {
-	int i;
 #ifndef UINPUT_OK
 	return 0;
 #else
+	int i;
 	if (UT.release) {
 		int maj, min;
 		/* guard against linux 2.4 */
@@ -455,6 +487,8 @@ static void ptr_move(int dx, int dy) {
 	ev.code = SYN_REPORT;
 	ev.value = 0;
 	write(fd, &ev, sizeof(ev));
+#else
+	if (!dx || !dy) {}
 #endif
 }
 
@@ -485,6 +519,8 @@ static void ptr_abs(int x, int y) {
 	ev.code = SYN_REPORT;
 	ev.value = 0;
 	write(fd, &ev, sizeof(ev));
+#else
+	if (!x || !y) {}
 #endif
 }
 
@@ -668,7 +704,8 @@ static void button_click(int down, int btn) {
 	write(fd, &ev, sizeof(ev));
 
 	last_button_click = dnow();
-
+#else
+	if (!down || !btn) {}
 #endif
 }
 
@@ -910,6 +947,8 @@ void uinput_key_command(int down, int keysym, rfbClientPtr client) {
 	if (0 <= scancode && scancode < 256) {
 		key_pressed[scancode] = down ? 1 : 0;
 	}
+#else
+	if (!down || !keysym || !client) {}
 #endif
 }
 
