@@ -1,7 +1,7 @@
 /*
  * classroom_manager.cpp - implementation of classroom-manager
  *
- * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ * Copyright (c) 2004-2010 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
  *
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -48,15 +48,14 @@
 #include <QtNetwork/QHostInfo>
 
 
-#include "main_window.h"
+#include "MainWindow.h"
 #include "classroom_manager.h"
 #include "client.h"
 #include "dialogs.h"
 #include "cmd_input_dialog.h"
-#include "italc_side_bar.h"
 #include "local_system.h"
 #include "tool_button.h"
-#include "tool_bar.h"
+#include "MainToolBar.h"
 #include "messagebox.h"
 
 #define DEFAULT_WINDOW_WIDTH	1005
@@ -85,9 +84,9 @@ QPixmap * classRoomItem::s_clientObservedPixmap = NULL;
 
 
 
-classroomManager::classroomManager( mainWindow * _main_window,
+classroomManager::classroomManager( MainWindow * _main_window,
 							QWidget * _parent ) :
-	sideBarWidget( QPixmap( ":/resources/classroom_manager.png" ),
+	SideBarWidget( QPixmap( ":/resources/classroom_manager.png" ),
 			tr( "Classroom-Manager" ),
 			tr( "Use this workspace to manage your computers and "
 				"classrooms in an easy way." ),
@@ -97,14 +96,14 @@ classroomManager::classroomManager( mainWindow * _main_window,
 	m_quickSwitchMenu( new QMenu( this ) ),
 	m_qsmClassRoomSeparator( m_quickSwitchMenu->addSeparator() ),
 	m_globalClientMode( client::Mode_Overview ),
-	m_clientUpdateInterval( 1 )
+	m_clientUpdateInterval( 1 ),
+	m_autoArranged( false )
 {
 	// some code called out of this function relies on m_classroomManager
 	// which actually is assigned after this function returns
 	_main_window->m_classroomManager = this;
 
-	QVBoxLayout * l = dynamic_cast<QVBoxLayout *>(
-						contentParent()->layout() );
+	QVBoxLayout * l = new QVBoxLayout( contentParent() );
 
 	m_view = new classTreeWidget( contentParent() );
 	l->addWidget( m_view );
@@ -135,8 +134,7 @@ classroomManager::classroomManager( mainWindow * _main_window,
 	QFont f;
 	f.setPixelSize( 12 );
 
-	m_showUsernameCheckBox = new QCheckBox( tr( "Show usernames" ),
-							contentParent() );
+	m_showUsernameCheckBox = new QCheckBox( tr( "Show usernames" ), contentParent() );
 	m_showUsernameCheckBox->setFont( f );
 	l->addWidget( m_showUsernameCheckBox );
 	connect( m_showUsernameCheckBox, SIGNAL( stateChanged( int ) ),
@@ -329,7 +327,7 @@ void classroomManager::saveGlobalClientConfig( void )
 	}
 
 	QString xml = "<?xml version=\"1.0\"?>\n" + doc.toString( 2 );
-/*	if( mainWindow::ensureConfigPathExists() == FALSE )
+/*	if( MainWindow::ensureConfigPathExists() == FALSE )
 	{
 		qFatal( QString( "Could not read/write or create directory %1!"
 					"For running iTALC, make sure you have "
@@ -366,17 +364,17 @@ void classroomManager::savePersonalConfig( void )
 	QDomElement globalsettings = doc.createElement( "globalsettings" );
 	globalsettings.setAttribute( "client-update-interval",
 						m_clientUpdateInterval );
-	globalsettings.setAttribute( "win-width", getMainWindow()->width() );
-	globalsettings.setAttribute( "win-height", getMainWindow()->height() );
-	globalsettings.setAttribute( "win-x", getMainWindow()->x() );
-	globalsettings.setAttribute( "win-y", getMainWindow()->y() );	
+	globalsettings.setAttribute( "win-width", mainWindow()->width() );
+	globalsettings.setAttribute( "win-height", mainWindow()->height() );
+	globalsettings.setAttribute( "win-x", mainWindow()->x() );
+	globalsettings.setAttribute( "win-y", mainWindow()->y() );	
 	globalsettings.setAttribute( "ismaximized",
-					getMainWindow()->isMaximized() );
+					mainWindow()->isMaximized() );
 	globalsettings.setAttribute( "opened-tab",
-				getMainWindow()->m_sideBar->openedTab() );
+				mainWindow()->sideBar()->activeTab() );
 
 	globalsettings.setAttribute( "wincfg", QString(
-				getMainWindow()->saveState().toBase64() ) );
+				mainWindow()->saveState().toBase64() ) );
 
 	globalsettings.setAttribute( "defaultdomain", __default_domain );
 	globalsettings.setAttribute( "demoquality", __demo_quality );
@@ -389,20 +387,22 @@ void classroomManager::savePersonalConfig( void )
 						m_clientDblClickAction );
 	globalsettings.setAttribute( "showUserColumn",
 						m_showUsernameCheckBox->isChecked() );
+	globalsettings.setAttribute( "autoarranged",
+					isAutoArranged() );
 
 	QStringList hidden_buttons;
-	foreach( QAction * a, getMainWindow()->getToolBar()->actions() )
+	foreach( QAction * a, mainWindow()->toolBar()->actions() )
 	{
 		if( !a->isVisible() )
 		{
 			hidden_buttons += a->text();
 		}
 	}
-	foreach( KMultiTabBarTab * tab, getMainWindow()->getSideBar()->tabs() )
+	foreach( QAbstractButton * btn, mainWindow()->sideBar()->tabs() )
 	{
-		if( !tab->isTabVisible() )
+		if( !btn->isVisible() )
 		{
-			hidden_buttons += tab->text();
+			hidden_buttons += btn->text();
 		}
 	}
 	globalsettings.setAttribute( "toolbarcfg", hidden_buttons.join( "#" ) );
@@ -426,7 +426,7 @@ void classroomManager::savePersonalConfig( void )
 	}
 
 	QString xml = "<?xml version=\"1.0\"?>\n" + doc.toString( 2 );
-	if( mainWindow::ensureConfigPathExists() == FALSE )
+	if( MainWindow::ensureConfigPathExists() == FALSE )
 	{
 		qWarning( QString( "Could not read/write or create directory "
 					"%1! For running iTALC, make sure you "
@@ -587,10 +587,10 @@ void classroomManager::getHeaderInformation( const QDomElement & _header )
 				node.toElement().attribute( "win-y" ) !=
 								QString::null )
 			{
-getMainWindow()->resize( node.toElement().attribute( "win-width" ).toInt(),
+mainWindow()->resize( node.toElement().attribute( "win-width" ).toInt(),
 			node.toElement().attribute("win-height" ).toInt() );
 
-getMainWindow()->move( node.toElement().attribute( "win-x" ).toInt(),
+mainWindow()->move( node.toElement().attribute( "win-x" ).toInt(),
 				node.toElement().attribute( "win-y" ).toInt() );
 			}
 			else
@@ -600,14 +600,14 @@ getMainWindow()->move( node.toElement().attribute( "win-x" ).toInt(),
 			if( node.toElement().attribute( "opened-tab" ) !=
 								QString::null )
 			{
-				getMainWindow()->m_openedTabInSideBar =
+				mainWindow()->m_openedTabInSideBar =
 						node.toElement().attribute(
 							"opened-tab" ).toInt();
 			}
 			if( node.toElement().attribute( "ismaximized" ).
 								toInt() > 0 )
 			{
-	getMainWindow()->setWindowState( getMainWindow()->windowState() |
+	mainWindow()->setWindowState( mainWindow()->windowState() |
 							Qt::WindowMaximized );
 			}
 			if( node.toElement().attribute( "wincfg" ) !=
@@ -622,6 +622,12 @@ getMainWindow()->move( node.toElement().attribute( "win-x" ).toInt(),
 				m_toolBarCfg = node.toElement().attribute(
 								"toolbarcfg" );
 			}
+			if( node.toElement().attribute( "autoarranged" ).
+								toInt() > 0 )
+			{
+				m_autoArranged = true;
+			}
+
 			__demo_quality = node.toElement().
 					attribute( "demoquality" ).toInt();
 
@@ -716,7 +722,7 @@ void classroomManager::loadTree( classRoom * _parent_item,
 						(client::types)e.attribute(
 							"type" ).toInt(),
 						_parent_item,
-						getMainWindow(),
+						mainWindow(),
 						e.attribute( "id" ).toInt() );
 				c->hide();
 			}
@@ -896,8 +902,8 @@ void classroomManager::loadGlobalClientConfig( void )
 
 void classroomManager::setDefaultWindowsSizeAndPosition( void )
 {
-	getMainWindow()->resize( DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT );
-	getMainWindow()->move( QPoint( 0, 0 ) );	
+	mainWindow()->resize( DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT );
+	mainWindow()->move( QPoint( 0, 0 ) );	
 }
 
 
@@ -1068,7 +1074,7 @@ void classroomManager::directSupport( void )
 	const QString h = supportDialog::getHost( this );
 	if( !h.isEmpty() )
 	{
-		getMainWindow()->remoteControlDisplay( h );
+		mainWindow()->remoteControlDisplay( h );
 	}
 }
 
@@ -1093,9 +1099,9 @@ void classroomManager::adjustWindows( void )
 	QVector<client *> vc = visibleClients();
 	if( vc.size() )
 	{
-		const int avail_w = getMainWindow()->workspace()->
+		const int avail_w = mainWindow()->workspace()->
 						parentWidget()->width();
-		const int avail_h = getMainWindow()->workspace()->
+		const int avail_h = mainWindow()->workspace()->
 						parentWidget()->height();
 		float cw = vc[0]->width() + decor_w;// add width of decoration
 		float ch = vc[0]->height() + decor_h;// add height of titlebar
@@ -1167,7 +1173,19 @@ void classroomManager::adjustWindows( void )
 			cl->move( static_cast<int>( cl->m_rasterX * nw )+1,
 				static_cast<int>( cl->m_rasterY * nh )+1 );
 		}
-		getMainWindow()->workspace()->updateGeometry();
+		mainWindow()->workspace()->updateGeometry();
+	}
+}
+
+
+
+
+void classroomManager::arrangeWindowsToggle( bool _on )
+{
+	m_autoArranged = _on;
+	if( _on == true )
+	{
+		arrangeWindows();
 	}
 }
 
@@ -1179,9 +1197,9 @@ void classroomManager::arrangeWindows( void )
 	QVector<client *> vc = visibleClients();
 	if( vc.size() )
 	{
-		const int avail_w = getMainWindow()->workspace()->
+		const int avail_w = mainWindow()->workspace()->
 						parentWidget()->width();
-		const int avail_h = getMainWindow()->workspace()->
+		const int avail_h = mainWindow()->workspace()->
 						parentWidget()->height();
 		const int w = avail_w;
 		const int h = avail_h;
@@ -1521,7 +1539,7 @@ void classroomManager::editClientSettings( void )
 		foreach( classRoomItem * cri, si )
 		{
 			clientSettingsDialog settings_dlg( cri->getClient(),
-					getMainWindow(),
+					mainWindow(),
 						cri->parent()->text( 0 ) );
 			settings_dlg.exec();
 		}
@@ -1744,7 +1762,7 @@ void classroomManager::addClient( void )
 		}
 	}
 
-	clientSettingsDialog settings_dlg( NULL, getMainWindow(),
+	clientSettingsDialog settings_dlg( NULL, mainWindow(),
 							classroom_name );
 	settings_dlg.setWindowTitle( tr( "Add computer" ) );
 	settings_dlg.exec();
@@ -1824,6 +1842,17 @@ void classroomManager::showUserColumn( int _show )
 
 
 
+void classroomManager::clientVisibleChanged( void )
+{
+	if( m_autoArranged == true )
+	{
+		arrangeWindows();
+	}
+}
+
+
+
+
 
 
 
@@ -2166,5 +2195,31 @@ void classRoomItem::setUser( const QString & _name )
 
 
 
+bool classRoomItem::operator<( const QTreeWidgetItem & other ) const
+{
+	int column = treeWidget()->sortColumn();
+	const QString str1 = text( column ).toLower();
+	const QString str2 = other.text( column ).toLower();
+	// check for numbered computers and sort them numerically correct (instead
+	// lexically correct) - example:
+	// * PC 1
+	// * PC 2
+	// * ...
+	// * PC 10
+	// * PC 11
+	// -> with lexical sorting, PC 10, PC 11... would appear after PC 1 and
+	// before PC 2, but we want PC 1, PC 2, ..., PC 10, PC 11
+	QRegExp rx1( "^([^\\d]*)(\\d+)$" );
+	QRegExp rx2( "^([^\\d]*)(\\d+)$" );
+	if( rx1.indexIn( str1 ) == 0 && rx2.indexIn( str2 ) == 0 &&
+			rx1.cap( 1 ) == rx2.cap( 1 ) )
+	{
+		return rx1.cap( 2 ).toInt() < rx2.cap( 2 ).toInt();
+	}
+	return str1 < str2;
+}
+
+
+
 #include "classroom_manager.moc"
 
