@@ -1,7 +1,7 @@
 /*
  * classroom_manager.h - include file for classroom-manager
  *
- * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ * Copyright (c) 2004-2010 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
  *
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -36,25 +36,26 @@
 #include <QtXml/QtXml>
 
 #include "client.h"
-#include "side_bar_widget.h"
+#include "SideBarWidget.h"
 
 
 
 class QButtonGroup;
 class QMenu;
 
+class classTreeWidget;
 class classRoom;
 class classRoomItem;
 class clientSettingsDialog;
 class configWidget;
-class mainWindow;
+class MainWindow;
 
 
-class classroomManager : public sideBarWidget
+class classroomManager : public SideBarWidget
 {
 	Q_OBJECT
 public:
-	classroomManager( mainWindow * _main_window, QWidget * _parent );
+	classroomManager( MainWindow * _main_window, QWidget * _parent );
 	virtual ~classroomManager();
 
 
@@ -72,8 +73,6 @@ public:
 
 	QVector<client *> getLoggedInClients( void ) const;
 
-	void createActionMenu( QMenu * _m );
-
 	inline int updateInterval( void ) const
 	{
 		return( m_clientUpdateInterval );
@@ -84,6 +83,11 @@ public:
 		return( m_winCfg );
 	}
 
+	inline const QString & toolBarCfg( void ) const
+	{
+		return( m_toolBarCfg );
+	}
+
 	client::modes globalClientMode( void ) const
 	{
 		return( m_globalClientMode );
@@ -105,25 +109,40 @@ public:
 		return( m_quickSwitchMenu );
 	}
 
+	bool showUsername( void ) const
+	{
+		return( m_showUsernameCheckBox->isChecked() );
+	}
+
 	void setStateOfClassRoom( classRoom * _cr, bool _shown );
 	QAction * addClassRoomToQuickSwitchMenu( classRoom * _cr );
 
+	void clientVisibleChanged( void );
+
+	void arrangeWindows( void );
+	bool isAutoArranged ( )
+	{
+		return m_autoArranged;
+	}
 
 public slots:
 	void updateClients( void );
 
+	// slots for context menu
+	void clientMenuRequest( void );
+	void clientMenuTriggered( QAction * _action );
+
 	// slots for toolbar-actions
 	void changeGlobalClientMode( int );
-	void logoutUser( void );
 	void sendMessage( void );
 	void powerOnClients( void );
 	void powerDownClients( void );
 	void remoteLogon( void );
-	void execCmds( void );
+	void directSupport( void );
 
 	// slots for actions in view-menu
 	void adjustWindows( void );
-	void arrangeWindows( void );
+	void arrangeWindowsToggle( bool _on );
 	void increaseClientSize( void );
 	void decreaseClientSize( void );
 
@@ -165,18 +184,17 @@ private slots:
 
 
 private:
-	void cmdToVisibleClients( client::clientCmds _cmd,
-					const QString & _u_data = CONFIRM_NO );
+	void setupMenus( void );
 
 	void saveSettingsOfChildren( QDomDocument & _doc, QDomElement & _root,
 						QTreeWidgetItem * _lvi,
 						bool _is_global_config );
-	void createActionMenu( QTreeWidgetItem * _p, QMenu * _m );
-	
+
 	void getHeaderInformation( const QDomElement & _header );
 	void loadTree( classRoom * _parentItem,
 					const QDomElement & _parentElement,
 					bool _is_global_config );
+	void loadMenuElement( QDomElement _e );
 
 	QVector<classRoomItem *> selectedItems( void );
 	void getSelectedItems( QTreeWidgetItem * _p,
@@ -185,17 +203,25 @@ private:
 
 	void resizeClients( const int _new_width );
 
+	void removeClassRoom( classRoom * _cr );
 
-	QTreeWidget * m_view;
+	classTreeWidget * m_view;
 
 	QVector<classRoom *> m_classRooms;
 	QVector<client *> m_clientsToRemove;
 	QVector<classRoom *> m_classRoomsToRemove;
-	QDomDocument m_domTree;
 
 	const QString m_personalConfiguration;
 	const QString m_globalClientConfiguration;
 
+	/* context menu: */
+	QActionGroup * m_classRoomItemActionGroup;
+	QActionGroup * m_classRoomActionGroup;
+	QActionGroup * m_contextActionGroup;
+
+	QMenu * m_clientMenu; /* template */
+	QVector<QDomNode> m_customMenuConfiguration;
+
 	QSpinBox * m_updateIntervalSpinBox;
 	QMenu * m_quickSwitchMenu;
 	QAction * m_qsmClassRoomSeparator;
@@ -203,6 +229,7 @@ private:
 
 	int m_clientUpdateInterval;
 	QString m_winCfg;
+	QString m_toolBarCfg;
 
 	int m_clientDblClickAction;
 
@@ -210,7 +237,39 @@ private:
 	friend class configWidget;
 
 	QPushButton * m_exportToFileBtn;
-	QCheckBox * m_showUserColumnCheckBox;
+	QCheckBox * m_showUsernameCheckBox;
+
+	bool m_autoArranged;
+} ;
+
+
+
+
+
+
+class classTreeWidget : public QTreeWidget
+{
+	Q_OBJECT
+public:
+	classTreeWidget( QWidget * _parent );
+	virtual ~classTreeWidget() { } ;
+
+	void setCurrentItem( QTreeWidgetItem * _item );
+
+private:
+	virtual void mousePressEvent( QMouseEvent * _me );
+	virtual void mouseMoveEvent( QMouseEvent * _me );
+	virtual void mouseReleaseEvent( QMouseEvent * _me );
+	virtual void dragMoveEvent( QDragMoveEvent * _e );
+	virtual void dropEvent( QDropEvent * _e );
+
+	bool droppingOnItself( QTreeWidgetItem * _target );
+
+	client * m_clientPressed;
+	QList<QTreeWidgetItem *> m_selectedItems;
+
+private slots:
+	void itemSelectionChanged( void );
 
 } ;
 
@@ -229,8 +288,6 @@ public:
 						QTreeWidgetItem * _parent );
 	virtual ~classRoom();
 
-	void createActionMenu( QMenu * _m, bool _add_sub_menu = TRUE );
-
 	void setMenuItemIcon( const QIcon & _icon )
 	{
 		m_qsMenuAction->setIcon( _icon );
@@ -239,13 +296,12 @@ public:
 		m_qsMenuAction->setFont( f );
 	}
 
-
 public slots:
 	void switchToClassRoom( void );
+	void clientMenuTriggered( QAction * _action );
 
 
 private slots:
-	void processCmdOnAllClients( QAction * _action );
 
 
 private:
@@ -271,14 +327,21 @@ public:
 	void setVisible( const bool _obs );
 	void setUser( const QString & _name );
 
+	inline bool isVisible( void ) const
+	{
+		return( m_visible );
+	}
+
 
 private:
-	classroomManager * m_classroomManager;
+	bool m_visible;
 	client * m_client;
 
 	static QPixmap * s_clientPixmap;
 	static QPixmap * s_clientObservedPixmap;
 
+	virtual bool operator<( const QTreeWidgetItem & other ) const;
+
 } ;
 
 
