@@ -2,7 +2,7 @@
  * client.cpp - code for client-windows, which are displayed in several
  *              instances in the main-window of iTALC
  *
- * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ * Copyright (c) 2004-2010 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
  *
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -33,7 +33,7 @@
 #include <QtGui/QMessageBox>
 
 
-#include "main_window.h"
+#include "MainWindow.h"
 #include "client.h"
 #include "ivs_connection.h"
 #include "classroom_manager.h"
@@ -215,8 +215,8 @@ void clientAction::process( QVector<client *> _clients, targetGroup _target )
 				remoteLogonDialog mld;
 
 				if( mld.exec() == QDialog::Accepted &&
-					!mld.userName().isEmpty() &&
-					!mld.password().isEmpty() )
+					!mld.userName().isEmpty()/* &&
+					!mld.password().isEmpty()*/ )
 				{
 					foreach( client * cl, _clients )
 					{
@@ -467,7 +467,7 @@ QMenu * clientMenu::createDefault( QWidget * _parent )
 client::client( const QString & _hostname,
 		const QString & _mac, const QString & _nickname,
 		types _type, classRoom * _class_room,
-					mainWindow * _main_window, int _id ) :
+					MainWindow * _main_window, int _id ) :
 	QWidget( _main_window->workspace() ),
 	m_mainWindow( _main_window ),
 	m_connection( NULL ),
@@ -477,7 +477,6 @@ client::client( const QString & _hostname,
 	m_nickname( _nickname ),
 	m_mac( _mac ),
 	m_type( _type ),
-	m_reloadsAfterReset( 0 ),
 	m_mode( Mode_Overview ),
 	m_user( "" ),
 	m_makeSnapshot( FALSE ),
@@ -718,7 +717,7 @@ bool client::userLoggedIn( void )
 	QMutexLocker ml( &m_syncMutex );
 
 	return( m_connection->state() == ivsConnection::Connected ||
-		m_connection->reset( m_hostname, &m_reloadsAfterReset ) ==
+		m_connection->reset( m_hostname ) ==
 						ivsConnection::Connected );
 }
 
@@ -757,6 +756,8 @@ void client::hideEvent( QHideEvent * )
 	{
 		m_classRoomItem->setVisible( FALSE );
 	}
+
+	m_mainWindow->getClassroomManager()->clientVisibleChanged();
 }
 
 
@@ -977,8 +978,11 @@ void client::paintEvent( QPaintEvent * _pe )
 				Qt::TextWordWrap | Qt::AlignCenter, msg );
 	QSize s( pm->size() );
 	s.scale( aw-10, ah-r.height()-20, Qt::KeepAspectRatio );
-	p.drawImage( ( aw-s.width() ) / 2, height()-ah,
+	if( s.width() > 0 && s.height() > 0 )
+	{
+		p.drawImage( ( aw-s.width() ) / 2, height()-ah,
 						fastQImage( *pm ).scaled( s ) );
+	}
 
 	p.setPen( QColor( 0, 0, 0 ) );
 	p.drawText( QRect( 5, height()-r.height()-10, aw - 10,
@@ -1019,6 +1023,8 @@ void client::showEvent( QShowEvent * )
 	{
 		m_classRoomItem->setVisible( TRUE );
 	}
+
+	m_mainWindow->getClassroomManager()->clientVisibleChanged();
 }
 
 
@@ -1030,18 +1036,8 @@ void client::reload()
 	{
 		m_syncMutex.lock();
 
-		// for some reason we must not send user-information-requests
-		// while first messages being exchanged between client and
-		// server
-		if( m_reloadsAfterReset > 5 )
-		{
-			m_connection->sendGetUserInformationRequest();
-		}
-		else
-		{
-			++m_reloadsAfterReset;
-		}
-		//if( m_connection->sendGetUserInformationRequest() )
+		m_connection->sendGetUserInformationRequest();
+		if( m_connection->sendGetUserInformationRequest() )
 		{
 			// only send a framebuffer-update-request if client
 			// is in (over)view-mode
@@ -1298,13 +1294,13 @@ void updateThread::update( void )
 	if( m_client->m_classRoomItem->isVisible() )
 	{
 		if( !m_client->m_mainWindow->remoteControlRunning() &&
-						!mainWindow::atExit() )
+						!MainWindow::atExit() )
 		{
 			m_client->reload();
 		}
 	}
 	else if( m_client->m_connection->state() == ivsConnection::Connected
-						&& !mainWindow::atExit() )
+						&& !MainWindow::atExit() )
 	{
 		m_client->m_connection->close();
 	}
