@@ -38,15 +38,16 @@
 #include <QtCore/QVector>
 #include <QtGui/QWidget>
 #include <QtGui/QImage>
+#include <QtGui/QMenu>
 
-class QMenu;
+#include "fast_qimage.h"
 
 class classRoom;
 class classRoomItem;
 class client;
 class isdConnection;
 class ivsConnection;
-class mainWindow;
+class MainWindow;
 
 
 typedef void( client:: * execCmd )( const QString & );
@@ -107,28 +108,102 @@ private:
 
 
 
+
+class clientAction : public QAction
+{
+	Q_OBJECT
+public:
+	enum type
+	{
+		Overview,
+		FullscreenDemo,
+		WindowDemo,
+		Locked,
+		ViewLive,
+		RemoteControl,
+		ClientDemo,
+		SendTextMessage,
+		LogonUser,
+		LogoutUser,
+		Snapshot,
+		PowerOn,
+		Reboot,
+		PowerDown,
+		ExecCmds,
+		RemoteScript,
+		LocalScript
+	} ;
+
+	enum targetGroup
+	{
+		Default,
+		SelectedClients,
+		VisibleClients
+	} ;
+
+	enum flags
+	{
+		None = 0,
+		FullMenu = 1
+	} ;
+
+	clientAction( type _type, QObject * _parent = 0, int _flags = 0 );
+	clientAction( type _type, const QIcon & _icon, const QString & _text,
+			QObject * _parent = 0, int _flags = 0 );
+	~clientAction() {};
+
+	void process( QVector<client *> _clients, targetGroup _target = Default );
+	static void process( QAction * _action,
+			QVector<client *> _clients, targetGroup _target = Default );
+
+	inline bool flags( int _mask = -1 )
+	{
+		return ( m_flags & _mask );
+	}
+
+private:
+	type m_type;
+	int m_flags;
+
+	bool confirmLogout( targetGroup _target ) const;
+	bool confirmReboot( targetGroup _target ) const;
+	bool confirmPowerDown( targetGroup _target ) const;
+	QString dataExpanded( QVector<client *> _clients ) const;
+
+} ;
+
+
+
+
+class clientMenu : public QMenu
+{
+	Q_OBJECT
+public:
+	static const bool FullMenu = TRUE;
+
+	clientMenu( const QString & _title, const QList<QAction *> _actions,
+			QWidget * _parent = 0, const bool _fullMenu = FALSE );
+
+	virtual ~clientMenu() {};
+
+	static QMenu * createDefault( QWidget * _parent );
+} ;
+
+
+
+
+inline QPixmap scaledIcon( const char * _name )
+{
+	return scaled( QString( ":/resources/" ) + _name, 16, 16 );
+}
+
+
+
+
 class client : public QWidget
 {
 	Q_OBJECT
 public:
-	enum clientCmds
-	{
-		Cmd_None,
-		Cmd_Reload,
-		Cmd_ViewLive,
-		Cmd_RemoteControl,
-		Cmd_ClientDemo,
-		Cmd_SendTextMessage,
-		Cmd_LogonUser,
-		Cmd_LogoutUser,
-		Cmd_Snapshot,
-		Cmd_PowerOn,
-		Cmd_Reboot,
-		Cmd_PowerDown,
-		Cmd_ExecCmds,
-		Cmd_CmdCount
-	} ;
-
 	enum modes
 	{
 		Mode_Overview,
@@ -155,13 +230,15 @@ public:
 		Type_Other
 	} ;
 
-	client( const QString & _local_ip,
+	client( const QString & _hostname,
 		const QString & _mac, const QString & _name, types _type,
-		classRoom * _class_room, mainWindow * _main_window,
+		classRoom * _class_room, MainWindow * _main_window,
 								int _id = -1 );
 
 	virtual ~client();
 
+	void quit( void );
+
 
 	int id( void ) const;
 	static client * clientFromID( int _id );
@@ -172,17 +249,36 @@ public:
 		return( m_mode );
 	}
 
-	void changeMode( const modes _new_mode, isdConnection * _conn );
+	// action-handlers
+	void changeMode( const modes _new_mode );
+	void viewLive( void );
+	void remoteControl( void );
+	void clientDemo( void );
+	void sendTextMessage( const QString & _msg );
+	void logonUser( const QString & _username, const QString & _password,
+			const QString & _domain );
+	void logoutUser( void );
+	void snapshot( void );
+	void powerOn( void );
+	void reboot( void );
+	void powerDown( void );
+	void execCmds( const QString & _cmds );
+	void reload( void );
 
 
 	inline QString name( void ) const
 	{
-		return( m_name );
+		return( m_nickname.isEmpty() ? m_hostname : m_nickname );
 	}
 
-	inline const QString & localIP( void ) const
+	inline const QString & hostname( void ) const
 	{
-		return( m_localIP );
+		return( m_hostname );
+	}
+
+	inline const QString & nickname( void ) const
+	{
+		return( m_nickname );
 	}
 
 	inline const QString & mac( void ) const
@@ -200,14 +296,14 @@ public:
 		return( m_user );
 	}
 
-	inline void setName( const QString & _name )
+	inline void setNickname( const QString & _nickname )
 	{
-		m_name = _name;
+		m_nickname = _nickname;
 	}
 
-	inline void setLocalIP( const QString & _local_ip )
+	inline void setHostname( const QString & _hostname )
 	{
-		m_localIP = _local_ip;
+		m_hostname = _hostname;
 	}
 
 	inline void setMac( const QString & _mac )
@@ -229,11 +325,6 @@ public:
 
 	virtual void update( void );
 
-	void createActionMenu( QMenu * _m );
-
-
-	void processCmd( clientCmds _cmd, const QString & _u_data = "" );
-
 	static inline bool reloadSnapshotList( void )
 	{
 		return( s_reloadSnapshotList );
@@ -245,29 +336,19 @@ public:
 	}
 
 
+	void zoom( void );
+	void zoomBack( void );
+
 
 	float m_rasterX;
 	float m_rasterY;
 
 
-	struct clientCommand
-	{
-		// stuff for cmd-exec
-		clientCmds	m_cmd;
-		execCmd 	m_exec;
-		// stuff for UI
-		char *		m_name;
-		char *		m_icon;
-		bool		m_insertSep;
-	} ;
-
-
-	static const clientCommand s_commands[Cmd_CmdCount];
-
-
-
 public slots:
-	void processCmdSlot( QAction * _action );
+
+
+private slots:
+	void enlarge( void );
 
 
 private:
@@ -284,33 +365,20 @@ private:
 	virtual void resizeEvent( QResizeEvent * _re );
 	virtual void showEvent( QShowEvent * _se );
 
-	// action-handlers
-	void reload( const QString & _update = CONFIRM_YES );
-	void clientDemo( const QString & );
-	void viewLive( const QString & );
-	void remoteControl( const QString & );
-	void sendTextMessage( const QString & _msg );
-	void logonUser( const QString & _uname_and_pw );
-	void logoutUser( const QString & _confirm );
-	void snapshot( const QString & );
-	void powerOn( const QString & );
-	void reboot( const QString & _confirm );
-	void powerDown( const QString & _confirm );
-	void execCmds( const QString & _cmds );
 
 	states currentState( void ) const;
 
 
-	mainWindow * m_mainWindow;
+	MainWindow * m_mainWindow;
 	ivsConnection * m_connection;
 	QPoint m_clickPoint;
 	QPoint m_origPos;
+	QSize m_origSize;
 
-	QString m_name;
-	QString m_localIP;
+	QString m_hostname;
+	QString m_nickname;
 	QString m_mac;
 	types m_type;
-	int m_reloadsAfterReset;
 
 	modes m_mode;
 	QString m_user;
@@ -322,8 +390,6 @@ private:
 
 	classRoomItem * m_classRoomItem;
 
-	QMenu * m_contextMenu;
-
 	updateThread * m_updateThread;
 
 
