@@ -1,7 +1,7 @@
 /*
  * config_widget.h - configuration-widget for side-bar
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ * Copyright (c) 2004-2010 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
  *
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -22,27 +22,26 @@
  *
  */
 
-
 #ifndef _CONFIG_WIDGET_H
 #define _CONFIG_WIDGET_H
 
-#include "side_bar_widget.h"
+#include "SideBarWidget.h"
 #include "dialogs/config.uic"
 
 
-class configWidget : public sideBarWidget, private Ui::config
+class configWidget : public SideBarWidget, private Ui::config
 {
 	Q_OBJECT
 public:
-	configWidget( mainWindow * _main_window, QWidget * _parent );
+	configWidget( MainWindow * _main_window, QWidget * _parent );
 	virtual ~configWidget();
 
 
 protected slots:
-	void interfaceSelected( const QString & _if_name );
 	void demoQualitySelected( int );
 	void roleSelected( int );
 	void toggleToolButtonTips( bool _on );
+	void toggleIconOnlyToolButtons( bool _on );
 	void domainChanged( const QString & _domain );
 
 } ;
