@@ -1,7 +1,7 @@
 /*
  * main.cpp - main-file for iTALC-Application
  *
- * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ * Copyright (c) 2004-2010 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
  *
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -22,14 +22,11 @@
  *
  */
 
-
-#include <QtCore/QLocale>
 #include <QtCore/QModelIndex>
-#include <QtCore/QTranslator>
 #include <QtGui/QApplication>
 #include <QtGui/QSplashScreen>
 
-#include "main_window.h"
+#include "MainWindow.h"
 #include "ivs_connection.h"
 #include "local_system_ima.h"
 #include "remote_control_widget.h"
@@ -69,31 +66,17 @@ int main( int argc, char * * argv )
 			"font-weight:bold; padding: 4px 32px 4px 20px; }" );
 #endif
 
-	// load translations
-	QString loc =  QLocale::system().name().toLower();
-	if( loc.left( 2 ) == loc.right( 2 ) )
-	{
-		loc = loc.left( 2 );
-	}
-
-	QTranslator app_tr;
-	app_tr.load( ":/resources/" + loc + ".qm" );
-	app.installTranslator( &app_tr );
-
-	QTranslator core_tr;
-	core_tr.load( ":/resources/" + loc + "-core.qm" );
-	app.installTranslator( &core_tr );
-
-	QTranslator qt_tr;
-	qt_tr.load( ":/resources/qt_" + loc + ".qm" );
-	app.installTranslator( &qt_tr );
-
-
 	qRegisterMetaType<QModelIndex>( "QModelIndex" );
 	qRegisterMetaType<quint16>( "quint16" );
 
 
 	localSystem::initialize();
+	if( QLocale::system().language() == QLocale::Hebrew ||
+		QLocale::system().language() == QLocale::Arabic )
+	{
+		QApplication::setLayoutDirection( Qt::RightToLeft );
+	}
+
 
 	__role = ISD::RoleTeacher;
 	if( localSystem::parameter( "isdport" ).toInt() > 0 )
@@ -167,20 +150,20 @@ int main( int argc, char * * argv )
 
 
 	// now create the main-window
-	mainWindow * main_window = new mainWindow( screen );
+	MainWindow * mainWindow = new MainWindow( screen );
 
-	if( !main_window->localISD() ||
-		main_window->localISD()->state() != isdConnection::Connected )
+	if( !mainWindow->localISD() ||
+		mainWindow->localISD()->state() != isdConnection::Connected )
 	{
-		return( -1 );
+		return -1;
 	}
 
 	// hide splash-screen as soon as main-window is shown
-	splashScreen->finish( main_window );
+	splashScreen->finish( mainWindow );
 
-	main_window->show();
+	mainWindow->show();
 
 	// let's rock!!
-	return( app.exec() );
+	return app.exec();
 }
 
