@@ -1,7 +1,7 @@
 /*
  * tool_button.cpp - implementation of iTALC-tool-button
  *
- * Copyright (c) 2006-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2006-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -32,6 +32,7 @@
 #include <QtGui/QLayout>
 #include <QtGui/QLinearGradient>
 #include <QtGui/QPainter>
+#include <QtGui/QToolBar>
 
 #include "tool_button.h"
 #include "fast_qimage.h"
@@ -42,54 +43,60 @@ const int MARGIN = 10;
 const int ROUNDED = 2000;
 
 bool toolButton::s_toolTipsDisabled = FALSE;
+bool toolButton::s_iconOnlyMode = FALSE;
 
 
 toolButton::toolButton( const QPixmap & _pixmap, const QString & _label,
 				const QString & _alt_label,
 				const QString & _title, 
 				const QString & _desc, QObject * _receiver, 
-				const char * _slot,	QWidget * _parent ) :
+				const char * _slot, QWidget * _parent ) :
 	QToolButton( _parent ),
 	m_pixmap( _pixmap ),
 	m_img( fastQImage( _pixmap.toImage() ).scaled( 32, 32 ) ),
-	m_colorizeLevel( 0 ),
-	m_fadeBack( FALSE ),
+	m_mouseOver( false ),
 	m_label( _label ),
 	m_altLabel( _alt_label ),
 	m_title( _title ),
 	m_descr( _desc )
 {
 	setAttribute( Qt::WA_NoSystemBackground, true );
-	QFont f = font();
-	f.setPointSize( 7 );
-	setFont( f );
-	//setFixedWidth( qMax<int>( m_img.width(), fontMetrics().width( m_hintLabel ) )+20 );
-	//setFixedHeight( 48 );
-	if( m_label.size() > 14 || m_altLabel.size() > 14 )
-	{
-		setFixedSize( 96, 48 );
-	}
-	else
-	{
-		setFixedSize( 88, 48 );
-	}
 
-	setText( m_title );
+	updateSize();
+
 	if( _receiver != NULL && _slot != NULL )
 	{
 		connect( this, SIGNAL( clicked() ), _receiver, _slot );
 	}
 
-	// set mask
-	QBitmap b( size() );
-	b.clear();
+}
+
+
+
+
+toolButton::toolButton( QAction * _a, const QString & _label,
+				const QString & _alt_label,
+				const QString & _desc, QObject * _receiver, 
+				const char * _slot, QWidget * _parent ) :
+	QToolButton( _parent ),
+	m_pixmap( _a->icon().pixmap( 128, 128 ) ),
+	m_img( fastQImage( m_pixmap.toImage() ).scaled( 32, 32 ) ),
+	m_mouseOver( false ),
+	m_label( _label ),
+	m_altLabel( _alt_label ),
+	m_title( _a->text() ),
+	m_descr( _desc )
+{
+	setAttribute( Qt::WA_NoSystemBackground, true );
+
+	updateSize();
+
+	if( _receiver != NULL && _slot != NULL )
+	{
+		connect( this, SIGNAL( clicked() ), _receiver, _slot );
+		connect( _a, SIGNAL( triggered( bool ) ), _receiver, _slot );
+	}
 
-	QPainter p( &b );
-	p.setBrush( Qt::color1 );
-	p.setPen( Qt::color1 );
-	p.drawRoundRect( 0, 0, width() - 1, height() - 1,
-					1000 / width(), 1000 / height() );
-	setMask( b );
 }
 
 
@@ -102,13 +109,31 @@ toolButton::~toolButton()
 
 
 
+void toolButton::setIconOnlyMode( bool _enabled )
+{
+	s_iconOnlyMode = _enabled;
+	QList<toolButton *> tbl = QApplication::activeWindow()->findChildren<toolButton *>();
+	foreach( toolButton * tb, tbl )
+	{
+		tb->updateSize();
+	}
+}
+
+
+
+
+void toolButton::addTo( QToolBar * _tb )
+{
+	QAction * a = _tb->addWidget( this );
+	a->setText( m_title );
+}
+
+
+
+
 void toolButton::enterEvent( QEvent * _e )
 {
-	m_fadeBack = FALSE;
-	if( m_colorizeLevel == 0 )
-	{
-		updateColorLevel();
-	}
+	m_mouseOver = true;
 	if( !s_toolTipsDisabled && !m_title.isEmpty() && !m_descr.isEmpty() )
 	{
 		QPoint p = mapToGlobal( QPoint( 0, 0 ) );
@@ -141,9 +166,10 @@ void toolButton::enterEvent( QEvent * _e )
 			p.setX( screen.x() );
 		if( p.y() + tbt->height() > screen.y() + screen.height() )
 			p.setY( screen.y() + screen.height() - tbt->height() );
-		tbt->move( p += QPoint( -4, 46 ) );
+		tbt->move( p += QPoint( -4, height() ) );
 		tbt->show();
 	}
+
 	QToolButton::enterEvent( _e );
 }
 
@@ -152,13 +178,10 @@ void toolButton::enterEvent( QEvent * _e )
 
 void toolButton::leaveEvent( QEvent * _e )
 {
-	emit mouseLeftButton();
-	m_fadeBack = TRUE;
-	if( m_colorizeLevel == 255 )
+	if( checkForLeaveEvent() )
 	{
-		updateColorLevel();
+		QToolButton::leaveEvent( _e );
 	}
-	QToolButton::leaveEvent( _e );
 }
 
 
@@ -176,102 +199,112 @@ void toolButton::mousePressEvent( QMouseEvent * _me )
 void toolButton::paintEvent( QPaintEvent * _pe )
 {
 	const bool active = isDown() || isChecked();
-	const QColor ctbl[2][4] = {
-				{
-					QColor( 64, 128, 255 ),
-					QColor( 32, 64, 192 ),
-					QColor( 16, 32, 128 ),
-					QColor( 0, 16, 64 )
-				},
-				{
-					QColor( 255, 224, 0, m_colorizeLevel ),
-					QColor( 224, 192, 0, m_colorizeLevel ),
-					QColor( 192, 160, 0, m_colorizeLevel ),
-					QColor( 96, 48, 0, m_colorizeLevel )
-				}
-				} ;
 
-	QPainter p( this );
-	p.fillRect( rect(), Qt::black );
-	p.setRenderHint( QPainter::Antialiasing );
-	QLinearGradient lingrad( 0, 0, 0, height() );
-	lingrad.setColorAt( 0, ctbl[0][0] );
-	lingrad.setColorAt( 0.38, ctbl[0][1] );
-	lingrad.setColorAt( 0.42, ctbl[0][2] );
-	lingrad.setColorAt( 1, ctbl[0][3] );
-	p.setBrush( lingrad );
-	p.drawRoundRect( 1, 1, width()-2, height()-2, 1000 / width(),
-							1000 / height() );
+	QPainter painter(this);
+	painter.setRenderHint(QPainter::SmoothPixmapTransform);
+	painter.setRenderHint(QPainter::Antialiasing);
+	painter.setPen(Qt::NoPen);
 
-	if( m_colorizeLevel )
+	QLinearGradient outlinebrush(0, 0, 0, height());
+	QLinearGradient brush(0, 0, 0, height());
+
+	brush.setSpread(QLinearGradient::PadSpread);
+	QColor highlight(255, 255, 255, 70);
+	QColor shadow(0, 0, 0, 70);
+	QColor sunken(220, 220, 220, 30);
+	QColor normal1(255, 255, 245, 60);
+	QColor normal2(255, 255, 235, 10);
+
+	if( active )
 	{
-		lingrad = QLinearGradient( 0, 0, 0, height() );
-		lingrad.setColorAt( 0, ctbl[1][0] );
-		lingrad.setColorAt( 0.38, ctbl[1][1] );
-		lingrad.setColorAt( 0.42, ctbl[1][2] );
-		lingrad.setColorAt( 1, ctbl[1][3] );
-		p.setBrush( lingrad );
-		p.drawRoundRect( 1, 1, width()-2, height()-2, 1000 / width(),
-							1000 / height() );
-	}
-	p.setBrush( QBrush() );
-
-	p.fillRect( rect(), QColor( 0, 0, 0, active ? 160 : 0 ) );
-
-	QPen pen( QColor( 255, 255, 255, 128 ) );
-	pen.setWidthF( 1.3f );
-	p.setPen( pen );
-	p.drawRoundRect( 0, 0, width()-1, height()-1, 1000 / width(),
-							1000 / height() );
-	QPen pen2 = pen;
-	pen.setColor( QColor( 0, 0, 0, 128 ) );
-	p.setPen( pen );
-	p.drawRoundRect( 1, 1, width()-3, height()-3, 1000 / width(),
-							1000 / height() );
-	p.setPen( pen2 );
-	p.drawRoundRect( 2, 2, width()-2, height()-2, 1000 / width(),
-							1000 / height() );
-
-	const int dd = isDown() ? 1 : 0;
-	const QPoint pt = QPoint( (width() - m_img.width() ) / 2 + dd, 3 + dd);
-	p.drawImage( pt, m_img );
-
-	const QString l = ( active && m_altLabel.isEmpty() == FALSE ) ?
-							m_altLabel : m_label;
-	const int w = p.fontMetrics().width( l );
-	p.setPen( Qt::black );
-	p.drawText( ( width() - w ) / 2 +1+dd, height() - 4+dd, l );
-	p.setPen( Qt::white );
-	p.drawText( ( width() - w ) / 2 +dd, height() - 5+dd, l );
-}
-
-
-
-
-
-void toolButton::updateColorLevel( void )
-{
-	bool again;
-	if( m_fadeBack )
-	{
-		m_colorizeLevel = qMax( 0, m_colorizeLevel - 10 );
-		again = m_colorizeLevel > 0;
+		outlinebrush.setColorAt(0.0f, shadow);
+		outlinebrush.setColorAt(1.0f, highlight);
+		brush.setColorAt(0.0f, sunken);
+		painter.setPen(Qt::NoPen);
 	}
 	else
 	{
-		m_colorizeLevel = qMin( 255, m_colorizeLevel + 10 );
-		again = m_colorizeLevel < 255;
+		outlinebrush.setColorAt(1.0f, shadow);
+		outlinebrush.setColorAt(0.0f, highlight);
+		brush.setColorAt(0.0f, normal1);
+		if( m_mouseOver == false )
+		{
+			brush.setColorAt(1.0f, normal2);
+		}
+		painter.setPen(QPen(outlinebrush, 1));
 	}
-	update();
-	if( again )
+
+	painter.setBrush(brush);
+
+	painter.drawRoundedRect( 0, 0, width(), height(), 5, 5 );
+
+	const int dd = active ? 1 : 0;
+	QPoint pt = QPoint( ( width() - m_img.width() ) / 2 + dd, 3 + dd );
+	if( s_iconOnlyMode )
 	{
-		QTimer::singleShot( 10, this, SLOT( updateColorLevel() ) );
+		pt.setY( ( height() - m_img.height() ) / 2 - 1 + dd );
+	}
+	painter.drawImage( pt, m_img );
+
+	if( s_iconOnlyMode == false )
+	{
+		const QString l = ( active && m_altLabel.isEmpty() == FALSE ) ?
+								m_altLabel : m_label;
+		const int w = painter.fontMetrics().width( l );
+		painter.setPen( Qt::black );
+		painter.drawText( ( width() - w ) / 2 +1+dd, height() - 4+dd, l );
+		painter.setPen( Qt::white );
+		painter.drawText( ( width() - w ) / 2 +dd, height() - 5+dd, l );
 	}
 }
 
 
 
 
+bool toolButton::checkForLeaveEvent()
+{
+	if( QRect( mapToGlobal( QPoint( 0, 0 ) ), size() ).
+					contains( QCursor::pos() ) )
+	{
+		QTimer::singleShot( 20, this, SLOT( checkForLeaveEvent() ) );
+	}
+	else
+	{
+		emit mouseLeftButton();
+		m_mouseOver = false;
+
+		return true;
+	}
+	return false;
+}
+
+
+
+
+void toolButton::updateSize()
+{
+	QFont f = font();
+	f.setPointSizeF( 8 );
+	setFont( f );
+
+	if( s_iconOnlyMode )
+	{
+		setFixedSize( 52, 48 );
+	}
+	else if( m_label.size() > 14 || m_altLabel.size() > 14 )
+	{
+		setFixedSize( 96, 48 );
+	}
+	else
+	{
+		setFixedSize( 88, 48 );
+	}
+}
+
+
+
+
+
 
 
 toolButtonTip::toolButtonTip( const QPixmap & _pixmap, const QString & _title,
@@ -281,7 +314,6 @@ toolButtonTip::toolButtonTip( const QPixmap & _pixmap, const QString & _title,
 	m_icon( fastQImage( _pixmap ).scaled( 72, 72 ) ),
 	m_title( _title ),
 	m_description( _description ),
-	m_dissolveSize( 24 ),
 	m_toolButton( _tool_btn )
 {
 	setAttribute( Qt::WA_DeleteOnClose, TRUE );
@@ -405,32 +437,8 @@ void toolButtonTip::updateMask( void )
 			p.fillRect( dx, 0, 10, 10, Qt::color1 );
 		}
 	}
-/*	p.setBrush( Qt::color0 );
-	p.setPen( Qt::color0 );
-
-	if( m_dissolveSize > 0 )
-	{
-		const int size = 16;
-		for( Q_UINT16 y = 0; y < height() + size; y += size )
-		{
-			Q_INT16 s = m_dissolveSize * width() / 128;
-			for( Q_INT16 x = width(); x > -size; x -= size, s -= 2 )
-			{
-				if( s < 0 )
-				{
-					s = 0;
-				}
-				p.drawEllipse( x - s / 2, y - s / 2, s, s );
-			}
-		}
-	}*/
 
 	setMask( b );
-
-/*	if( --m_dissolveSize >= 0 )
-	{
-		QTimer::singleShot( 20, this, SLOT( updateMask() ) );
-	}*/
 }
 
 
