@@ -2,7 +2,7 @@
  * ivs_connection.cpp - class ivsConnection, an implementation of the 
  *                      RFB-protocol with iTALC-extensions for Qt
  *
- * Copyright (c) 2004-2008 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ * Copyright (c) 2004-2010 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
  *
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -38,7 +38,7 @@
 
 
 #include "ivs_connection.h"
-#include "qt_user_events.h"
+#include "QuadTree.h"
 #include "local_system.h"
 #include "messagebox.h"
 
@@ -97,6 +97,8 @@ ivsConnection::ivsConnection( const QString & _host, quality _q,
 	m_zlibStreamActive[0] = m_zlibStreamActive[1] = m_zlibStreamActive[2] =
 						m_zlibStreamActive[3] = FALSE;
 #endif
+
+	qRegisterMetaType<RectList>( "RectList" );
 }
 
 
@@ -177,6 +179,7 @@ ivsConnection::states ivsConnection::protocolInitialization( void )
 
 	if( !readFromServer( desktop_name, m_si.nameLength ) )
 	{
+		delete[] desktop_name;
 		return( state_ref() = ConnectionFailed );
 	}
 
@@ -268,7 +271,7 @@ ivsConnection::states ivsConnection::protocolInitialization( void )
 
 	m_screen = QImage( m_si.framebufferWidth, m_si.framebufferHeight,
 							QImage::Format_RGB32 );
-
+	m_screen.fill( Qt::black );
 
 	sendFramebufferUpdateRequest();
 	sendGetUserInformationRequest();
@@ -292,13 +295,23 @@ void ivsConnection::close( void )
 
 bool ivsConnection::takeSnapshot( void )
 {
-	if( user().isEmpty() || state() != ivsConnection::Connected )
+	if( state() != ivsConnection::Connected )
 	{
 		return( FALSE );
 	}
 
+	QString u = user();
+	if( u.isEmpty() )
+	{
+		u = tr( "unknown" );
+	}
+	if( !u.contains( '(' ) )
+	{
+		u = QString( "%1 (%2)" ).arg( u ).arg( u );
+	}
+
 	// construct text
-	QString txt = user() + "@" + host() + " " +
+	QString txt = u + "@" + host() + " " +
 			QDate( QDate::currentDate() ).toString( Qt::ISODate ) +
 			" " + QTime( QTime::currentTime() ).
 							toString( Qt::ISODate );
@@ -317,7 +330,7 @@ bool ivsConnection::takeSnapshot( void )
 				"_" + QTime( QTime::currentTime() ).
 					toString( Qt::ISODate ) + ".png";
 	file_name.replace( ':', '-' );
-	file_name = dir + user().section( '(', 1, 1 ).section( ')', 0, 0 ) +
+	file_name = dir + u.section( '(', 1, 1 ).section( ')', 0, 0 ) +
 								file_name;
 	const int FONT_SIZE = 14;
 	const int RECT_MARGIN = 10;
@@ -444,11 +457,11 @@ bool ivsConnection::sendKeyEvent( Q_UINT32 key, bool down )
 
 
 
-void ivsConnection::postRegionChangedEvent( const QRegion & _rgn )
+void ivsConnection::postRegionChangedEvent( const RectList &rgn )
 {
 	if( parent() != NULL )
 	{
-		regionChangedEvent * rche = new regionChangedEvent( _rgn );
+		regionChangedEvent * rche = new regionChangedEvent( rgn );
 		QCoreApplication::postEvent( parent(), rche );
 	}
 }
@@ -466,7 +479,14 @@ void ivsConnection::rescaleScreen( void )
 			m_scaledScreen = QImage( m_scaledSize,
 							QImage::Format_RGB32 );
 		}
-		m_screen.scaleTo( m_scaledScreen );
+		if( m_screen.size().isValid() )
+		{
+			m_screen.scaleTo( m_scaledScreen );
+		}
+		else
+		{
+			m_scaledScreen.fill( Qt::black );
+		}
 		m_scaledScreenNeedsUpdate = FALSE;
 	}
 }
@@ -506,10 +526,10 @@ bool ivsConnection::handleServerMessages( bool _send_screen_update, int _tries )
 
 			msg.fu.nRects = swap16IfLE( msg.fu.nRects );
 
-			QRegion updated_region;
+			RectList updated_region;
 
 			rfbFramebufferUpdateRectHeader rect;
-			for( Q_UINT16 i = 0; i < msg.fu.nRects; i++ )
+			for( int i = 0; i < msg.fu.nRects; i++ )
 			{
 				if( !readFromServer( (char *)&rect,
 							sizeof( rect ) ) )
@@ -635,7 +655,8 @@ bool ivsConnection::handleServerMessages( bool _send_screen_update, int _tries )
 
 					case rfbEncodingItalcCursor:
 	{
-		QRegion ch_reg = QRect( m_cursorPos - m_cursorHotSpot,
+		RectList ch_reg;
+		ch_reg += QRect( m_cursorPos - m_cursorHotSpot,
 							m_cursorShape.size() );
 		m_cursorLock.lockForWrite();
 		//m_cursorShape = socketDev().read().value<QImage>();
@@ -680,15 +701,16 @@ bool ivsConnection::handleServerMessages( bool _send_screen_update, int _tries )
 				// if we're providing data for demo-purposes,
 				// we perform a simple color-reduction for
 				// better compression-results
-	const QVector<QRect> rects = updated_region.rects();
-	for( QVector<QRect>::const_iterator it = rects.begin();
+	QuadTree q( 0, 0, m_screen.width()-1, m_screen.height()-1, 4 );
+	q.addRects( updated_region );
+	const QVector<QuadTreeRect> rects = q.rects();
+	for( QVector<QuadTreeRect>::const_iterator it = rects.begin();
 					it != rects.end(); ++it )
 	{
-		for( Q_UINT16 y = 0; y < it->height(); ++y )
+		for( int y = it->y1(); y <= it->y2(); ++y )
 		{
-			QRgb * data = ( (QRgb *) m_screen.scanLine( y +
-							it->y() ) ) + it->x();
-			for( Q_UINT16 x = 0; x < it->width(); ++x )
+			QRgb * data = ( (QRgb *) m_screen.scanLine( y ) );
+			for( Q_UINT16 x = it->x1(); x <= it->x2(); ++x )
 			{
 				data[x] &= and_value;
 			}
@@ -1536,7 +1558,8 @@ bool ivsConnection::decompressJpegRect( Q_UINT16 x, Q_UINT16 y, Q_UINT16 w,
 bool ivsConnection::handleCursorPos( const Q_UINT16 _x, const Q_UINT16 _y )
 {
 	// move cursor and update appropriate region
-	QRegion ch_reg = QRect( m_cursorPos - m_cursorHotSpot,
+	RectList ch_reg;
+	ch_reg += QRect( m_cursorPos - m_cursorHotSpot,
 							m_cursorShape.size() );
 	m_cursorPos = QPoint( _x, _y );
 	ch_reg += QRect( m_cursorPos - m_cursorHotSpot, m_cursorShape.size() );
@@ -1690,7 +1713,8 @@ bool ivsConnection::handleCursorShape( const Q_UINT16 _xhot,
 	}
 
 
-	QRegion ch_reg = QRect( m_cursorPos - m_cursorHotSpot,
+	RectList ch_reg;
+	ch_reg += QRect( m_cursorPos - m_cursorHotSpot,
 							m_cursorShape.size() );
 	m_cursorLock.lockForWrite();
 	m_cursorShape = QImage( rcSource, _width, _height,
@@ -1750,13 +1774,15 @@ bool ivsConnection::handleItalc( Q_UINT16 rx, Q_UINT16 ry, Q_UINT16 rw,
 
 	Q_UINT8 * rle_data = new Q_UINT8[hdr.bytesRLE];
 
-	lzo_uint decomp_bytes = 0;
-	lzo1x_decompress( (const unsigned char *) lzo_data,
+	lzo_uint decomp_bytes = hdr.bytesRLE;
+	lzo1x_decompress_safe( (const unsigned char *) lzo_data,
 				(lzo_uint) hdr.bytesLZO,
 				(unsigned char *) rle_data,
 				(lzo_uint *) &decomp_bytes, NULL );
 	if( decomp_bytes != hdr.bytesRLE )
 	{
+		delete[] rle_data;
+		delete[] lzo_data;
 		qCritical( "ivsConnection::handleItalc(...): expected and real "
 				"size of decompressed data do not match!" );
 		return( FALSE );
