@@ -2,7 +2,7 @@
  * local_system.cpp - namespace localSystem, providing an interface for
  *                    transparent usage of operating-system-specific functions
  *
- * Copyright (c) 2006-2008 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ * Copyright (c) 2006-2010 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
  *
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -23,18 +23,22 @@
  *
  */
 
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#ifdef BUILD_WIN32
+#define _WIN32_WINNT 0x0501
+#endif
 
 #include <QtCore/QCoreApplication>
+#include <QtCore/QDateTime>
 #include <QtCore/QDir>
+#include <QtCore/QLocale>
 #include <QtCore/QMutex>
 #include <QtCore/QProcess>
 #include <QtCore/QSettings>
-#include <QtCore/QDateTime>
+#include <QtCore/QTranslator>
 #include <QtGui/QWidget>
 #include <QtNetwork/QTcpServer>
 
@@ -51,7 +55,6 @@ static const char * tr_accels = QT_TRANSLATE_NOOP(
 		"please keep this note as otherwise there are strange errors "
 					"concerning logon-feature)" );
 
-#define _WIN32_WINNT 0x0501
 #include <windows.h>
 #include <shlobj.h>
 #include <psapi.h>
@@ -72,13 +75,50 @@ QString windowsConfigPath( int _type )
 				library.resolve( "SHGetSpecialFolderPathA" );
 	if( SHGetSpecialFolderPath )
 	{
-	    char path[MAX_PATH];
-	    SHGetSpecialFolderPath( 0, path, _type, FALSE );
-	    result = QString::fromLocal8Bit( path );
+		char path[MAX_PATH];
+		SHGetSpecialFolderPath( 0, path, _type, FALSE );
+		result = QString::fromLocal8Bit( path );
 	}
 	return( result );
 }
 
+void killWisPtis()
+{
+	DWORD aProcesses[1024], cbNeeded;
+	if( !EnumProcesses( aProcesses, sizeof( aProcesses ), &cbNeeded ) )
+	{
+		return;
+	}
+
+	DWORD cProcesses = cbNeeded / sizeof(DWORD);
+
+	for( DWORD i = 0; i < cProcesses; i++ )
+	{
+		HANDLE hProcess = OpenProcess(
+							PROCESS_QUERY_INFORMATION | PROCESS_TERMINATE|
+								PROCESS_VM_READ,
+							false, aProcesses[i] );
+		HMODULE hMod;
+		if( hProcess == NULL ||
+			!EnumProcessModules( hProcess, &hMod, sizeof( hMod ),
+								&cbNeeded ) )
+		{
+			continue;
+		}
+		TCHAR szProcessName[MAX_PATH];
+		GetModuleBaseName( hProcess, hMod, szProcessName,
+							sizeof(szProcessName)/sizeof(TCHAR) );
+		for( TCHAR * ptr = szProcessName; *ptr; ++ptr )
+		{
+			*ptr = tolower( *ptr );
+		}
+		if( strcmp( szProcessName, "wisptis.exe" ) == 0 )
+		{
+			TerminateProcess( hProcess, 0 );
+		}
+	}
+}
+
 }
 
 #endif
@@ -119,7 +159,7 @@ static QString properLineEnding( QString _out )
 {
 	if( _out.right( 1 ) != "\012" )
 	{
-		_out += "\012";				
+		_out += "\012";
 	}
 #ifdef BUILD_WIN32
 	if( _out.right( 1 ) != "\015" )
@@ -132,7 +172,7 @@ static QString properLineEnding( QString _out )
 	{
 		_out.replace( QString( "\012" ), QString( "\015" ) );
 	}
-#endif			
+#endif
 	return( _out );
 }
 
@@ -229,11 +269,6 @@ void msgHandler( QtMsgType _type, const char * _msg )
 }
 
 
-void initResources( void )
-{
-	Q_INIT_RESOURCE(italc_core);
-}
-
 namespace localSystem
 {
 
@@ -242,6 +277,9 @@ int IC_DllExport logLevel = 6;
 
 void initialize( p_pressKey _pk, const QString & _log_file )
 {
+#ifdef BUILD_WIN32
+	killWisPtis();
+#endif
 	__pressKey = _pk;
 	__log_file = _log_file;
 
@@ -252,7 +290,7 @@ void initialize( p_pressKey _pk, const QString & _log_file )
 	QCoreApplication::setApplicationName( "iTALC" );
 
 	QSettings settings( QSettings::SystemScope, "iTALC Solutions", "iTALC" );
-	
+
 	if( settings.contains( "settings/LogLevel" ) )
 	{
 		logLevel = settings.value( "settings/LogLevel" ).toInt();
@@ -260,7 +298,21 @@ void initialize( p_pressKey _pk, const QString & _log_file )
 
 	qInstallMsgHandler( msgHandler );
 
-	initResources();
+	// load translations
+	QString loc = QLocale::system().name().left( 2 );
+	if( loc.left( 2 ) == loc.right( 2 ) )
+	{
+		loc = loc.left( 2 );
+	}
+
+	static QTranslator appTr;
+	appTr.load( ":/resources/" + loc + ".qm" );
+	QCoreApplication::installTranslator( &appTr );
+
+	static QTranslator qtTr;
+	qtTr.load( ":/resources/qt_" + loc + ".qm" );
+	QCoreApplication::installTranslator( &qtTr );
+
 }
 
 
@@ -315,7 +367,7 @@ void broadcastWOLPacket( const QString & _mac )
 	unsigned char mac[MAC_SIZE];
 	char out_buf[OUTBUF_SIZE];
 
-	if( sscanf( _mac.toAscii().constData(),
+	if( sscanf( _mac.toUtf8().constData(),
 				"%2x:%2x:%2x:%2x:%2x:%2x",
 				(unsigned int *) &mac[0],
 				(unsigned int *) &mac[1],
@@ -330,7 +382,7 @@ void broadcastWOLPacket( const QString & _mac )
 
 	for( int i = 0; i < MAC_SIZE; ++i )
 	{
-		out_buf[i] = 0xff;	  
+		out_buf[i] = 0xff;
 	}
 
 	for( int i = 1; i < 17; ++i )
@@ -419,12 +471,12 @@ void logonUser( const QString & _uname, const QString & _passwd,
 		if( hProcess == NULL ||
 			!EnumProcessModules( hProcess, &hMod, sizeof( hMod ),
 								&cbNeeded ) )
-	        {
+		{
 			continue;
 		}
 		TCHAR szProcessName[MAX_PATH];
-		GetModuleBaseName( hProcess, hMod, szProcessName, 
-                             		  sizeof(szProcessName)/sizeof(TCHAR) );
+		GetModuleBaseName( hProcess, hMod, szProcessName,
+							sizeof(szProcessName)/sizeof(TCHAR) );
 		for( TCHAR * ptr = szProcessName; *ptr; ++ptr )
 		{
 			*ptr = tolower( *ptr );
@@ -445,12 +497,12 @@ void logonUser( const QString & _uname, const QString & _passwd,
 	enablePrivilege( SE_TCB_NAME, TRUE );
 	HANDLE hToken;
 	if( !LogonUser(
-		(CHAR *)_uname.toAscii().constData(),
-		(CHAR*)(_domain.isEmpty() ? "." : _domain.toAscii().constData()),
-		(CHAR *)_passwd.toAscii().constData(),
+		(CHAR *)_uname.toUtf8().constData(),
+		(CHAR*)(_domain.isEmpty() ? "." : _domain.toUtf8().constData()),
+		(CHAR *)_passwd.toUtf8().constData(),
 		LOGON32_LOGON_INTERACTIVE,
 		LOGON32_PROVIDER_DEFAULT,
-		&hToken ) ) 
+		&hToken ) )
 	{
 		CloseHandle( hToken );
 	}
@@ -460,9 +512,9 @@ void logonUser( const QString & _uname, const QString & _passwd,
 	if( GetKeyState( VK_CAPITAL ) & 1 )
 	{
 		INPUT input[2];
-		ZeroMemory( input, sizeof( input ) );        
+		ZeroMemory( input, sizeof( input ) );
 		input[0].type = input[1].type = INPUT_KEYBOARD;
-		input[0].ki.wVk = input[1].ki.wVk = VK_CAPITAL;        
+		input[0].ki.wVk = input[1].ki.wVk = VK_CAPITAL;
 		input[1].ki.dwFlags = KEYEVENTF_KEYUP;
 		SendInput( 2, input, sizeof( INPUT ) );
 	}
@@ -678,7 +730,7 @@ QString globalConfigPath( void )
 QString personalConfigDir( void )
 {
 	QSettings settings;
-	const QString d = settings.value( "paths/personalconfig" ).toString();
+	const QString d = settings.value( "paths/personalconfigdir" ).toString();
 	return( d.isEmpty() ?
 #ifdef BUILD_WIN32
 				windowsConfigPath( CSIDL_APPDATA ) +
@@ -689,7 +741,7 @@ QString personalConfigDir( void )
 #endif
 				+ QDir::separator()
 		:
-				d );
+				d + QDir::separator() );
 }
 
 
