@@ -1,7 +1,7 @@
 /*
- * main.cpp - main-file for iTALC-setup-application
+ * main.cpp - main file for iTALC setup tool
  *
- * Copyright (c) 2006-2007 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
+ * Copyright (c) 2006-2010 Tobias Doerffel <tobydox/at/users/dot/sf/dot/net>
  *
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -22,10 +22,7 @@
  *
  */
 
-
 #include <QtCore/QFileInfo>
-#include <QtCore/QLocale>
-#include <QtCore/QTranslator>
 #include <QtCore/QDir>
 #include <QtGui/QApplication>
 #include <QtGui/QMessageBox>
@@ -44,18 +41,12 @@ int main( int argc, char * * argv )
 
 	localSystem::initialize( NULL, "italc_setup.log" );
 
-	// load translations
-	const QString loc = QLocale::system().name().left( 2 );
-
-	QTranslator app_tr;
-	app_tr.load( ":/resources/" + loc + ".qm" );
-	app.installTranslator( &app_tr );
-
-	QTranslator qt_tr;
-	qt_tr.load( ":/resources/qt_" + loc + ".qm" );
-	app.installTranslator( &qt_tr );
-
-	setupWizard sw;
+	QString installDir = QCoreApplication::applicationDirPath();
+	if( app.arguments().size() > 1 && QFileInfo( app.arguments()[1] ).isDir() )
+	{
+		installDir = app.arguments()[1];
+	}
+	setupWizard sw( installDir );
 
 	if( app.arguments().size() > 1 )
 	{
@@ -79,8 +70,8 @@ int main( int argc, char * * argv )
 		else
 		{
 			QMessageBox::information( NULL,
-					app.tr( "File does not exist" ),
-					app.tr( "The file %1 could not be "
+					setupWizard::tr( "File does not exist" ),
+					setupWizard::tr( "The file %1 could not be "
 						"found. Please check this and "
 						"try again." ).
 						arg( app.arguments()[1] ) );
@@ -91,7 +82,6 @@ int main( int argc, char * * argv )
 		sw.show();
 	}
 
-	// let's rock!!
-	return( app.exec() );
+	return app.exec();
 }
 
